import datetime
from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..extensions import NotPresentError, UnknownType
from ..models.fields import Fields
from ..models.request_creator import RequestCreator
from ..models.request_requestor import RequestRequestor
from ..models.request_sample_group import RequestSampleGroup
from ..models.request_schema_property import RequestSchemaProperty
from ..models.request_status import RequestStatus
from ..models.request_task import RequestTask
from ..models.request_team_assignee import RequestTeamAssignee
from ..models.request_user_assignee import RequestUserAssignee
from ..types import UNSET, Unset

T = TypeVar("T", bound="Request")


@attr.s(auto_attribs=True, repr=False)
class Request:
    """  """

    _api_url: Union[Unset, str] = UNSET
    _assignees: Union[Unset, List[Union[RequestUserAssignee, RequestTeamAssignee, UnknownType]]] = UNSET
    _created_at: Union[Unset, str] = UNSET
    _creator: Union[Unset, RequestCreator] = UNSET
    _display_id: Union[Unset, str] = UNSET
    _fields: Union[Unset, Fields] = UNSET
    _id: Union[Unset, str] = UNSET
    _project_id: Union[Unset, str] = UNSET
    _request_status: Union[Unset, RequestStatus] = UNSET
    _requestor: Union[Unset, RequestRequestor] = UNSET
    _sample_groups: Union[Unset, List[RequestSampleGroup]] = UNSET
    _scheduled_on: Union[Unset, None, datetime.date] = UNSET
    _schema: Union[Unset, RequestSchemaProperty] = UNSET
    _tasks: Union[Unset, List[RequestTask]] = UNSET
    _web_url: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("api_url={}".format(repr(self._api_url)))
        fields.append("assignees={}".format(repr(self._assignees)))
        fields.append("created_at={}".format(repr(self._created_at)))
        fields.append("creator={}".format(repr(self._creator)))
        fields.append("display_id={}".format(repr(self._display_id)))
        fields.append("fields={}".format(repr(self._fields)))
        fields.append("id={}".format(repr(self._id)))
        fields.append("project_id={}".format(repr(self._project_id)))
        fields.append("request_status={}".format(repr(self._request_status)))
        fields.append("requestor={}".format(repr(self._requestor)))
        fields.append("sample_groups={}".format(repr(self._sample_groups)))
        fields.append("scheduled_on={}".format(repr(self._scheduled_on)))
        fields.append("schema={}".format(repr(self._schema)))
        fields.append("tasks={}".format(repr(self._tasks)))
        fields.append("web_url={}".format(repr(self._web_url)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "Request({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        api_url = self._api_url
        assignees: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._assignees, Unset):
            assignees = []
            for assignees_item_data in self._assignees:
                if isinstance(assignees_item_data, UnknownType):
                    assignees_item = assignees_item_data.value
                elif isinstance(assignees_item_data, RequestUserAssignee):
                    assignees_item = assignees_item_data.to_dict()

                else:
                    assignees_item = assignees_item_data.to_dict()

                assignees.append(assignees_item)

        created_at = self._created_at
        creator: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self._creator, Unset):
            creator = self._creator.to_dict()

        display_id = self._display_id
        fields: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self._fields, Unset):
            fields = self._fields.to_dict()

        id = self._id
        project_id = self._project_id
        request_status: Union[Unset, int] = UNSET
        if not isinstance(self._request_status, Unset):
            request_status = self._request_status.value

        requestor: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self._requestor, Unset):
            requestor = self._requestor.to_dict()

        sample_groups: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._sample_groups, Unset):
            sample_groups = []
            for sample_groups_item_data in self._sample_groups:
                sample_groups_item = sample_groups_item_data.to_dict()

                sample_groups.append(sample_groups_item)

        scheduled_on: Union[Unset, None, str] = UNSET
        if not isinstance(self._scheduled_on, Unset):
            scheduled_on = self._scheduled_on.isoformat() if self._scheduled_on else None

        schema: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self._schema, Unset):
            schema = self._schema.to_dict()

        tasks: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._tasks, Unset):
            tasks = []
            for tasks_item_data in self._tasks:
                tasks_item = tasks_item_data.to_dict()

                tasks.append(tasks_item)

        web_url = self._web_url

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if api_url is not UNSET:
            field_dict["apiURL"] = api_url
        if assignees is not UNSET:
            field_dict["assignees"] = assignees
        if created_at is not UNSET:
            field_dict["createdAt"] = created_at
        if creator is not UNSET:
            field_dict["creator"] = creator
        if display_id is not UNSET:
            field_dict["displayId"] = display_id
        if fields is not UNSET:
            field_dict["fields"] = fields
        if id is not UNSET:
            field_dict["id"] = id
        if project_id is not UNSET:
            field_dict["projectId"] = project_id
        if request_status is not UNSET:
            field_dict["requestStatus"] = request_status
        if requestor is not UNSET:
            field_dict["requestor"] = requestor
        if sample_groups is not UNSET:
            field_dict["sampleGroups"] = sample_groups
        if scheduled_on is not UNSET:
            field_dict["scheduledOn"] = scheduled_on
        if schema is not UNSET:
            field_dict["schema"] = schema
        if tasks is not UNSET:
            field_dict["tasks"] = tasks
        if web_url is not UNSET:
            field_dict["webURL"] = web_url

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_api_url() -> Union[Unset, str]:
            api_url = d.pop("apiURL")
            return api_url

        try:
            api_url = get_api_url()
        except KeyError:
            if strict:
                raise
            api_url = cast(Union[Unset, str], UNSET)

        def get_assignees() -> Union[
            Unset, List[Union[RequestUserAssignee, RequestTeamAssignee, UnknownType]]
        ]:
            assignees = []
            _assignees = d.pop("assignees")
            for assignees_item_data in _assignees or []:

                def _parse_assignees_item(
                    data: Union[Dict[str, Any]]
                ) -> Union[RequestUserAssignee, RequestTeamAssignee, UnknownType]:
                    assignees_item: Union[RequestUserAssignee, RequestTeamAssignee, UnknownType]
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        assignees_item = RequestUserAssignee.from_dict(data, strict=True)

                        return assignees_item
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        assignees_item = RequestTeamAssignee.from_dict(data, strict=True)

                        return assignees_item
                    except:  # noqa: E722
                        pass
                    return UnknownType(data)

                assignees_item = _parse_assignees_item(assignees_item_data)

                assignees.append(assignees_item)

            return assignees

        try:
            assignees = get_assignees()
        except KeyError:
            if strict:
                raise
            assignees = cast(
                Union[Unset, List[Union[RequestUserAssignee, RequestTeamAssignee, UnknownType]]], UNSET
            )

        def get_created_at() -> Union[Unset, str]:
            created_at = d.pop("createdAt")
            return created_at

        try:
            created_at = get_created_at()
        except KeyError:
            if strict:
                raise
            created_at = cast(Union[Unset, str], UNSET)

        def get_creator() -> Union[Unset, RequestCreator]:
            creator: Union[Unset, Union[Unset, RequestCreator]] = UNSET
            _creator = d.pop("creator")

            if not isinstance(_creator, Unset):
                creator = RequestCreator.from_dict(_creator)

            return creator

        try:
            creator = get_creator()
        except KeyError:
            if strict:
                raise
            creator = cast(Union[Unset, RequestCreator], UNSET)

        def get_display_id() -> Union[Unset, str]:
            display_id = d.pop("displayId")
            return display_id

        try:
            display_id = get_display_id()
        except KeyError:
            if strict:
                raise
            display_id = cast(Union[Unset, str], UNSET)

        def get_fields() -> Union[Unset, Fields]:
            fields: Union[Unset, Union[Unset, Fields]] = UNSET
            _fields = d.pop("fields")

            if not isinstance(_fields, Unset):
                fields = Fields.from_dict(_fields)

            return fields

        try:
            fields = get_fields()
        except KeyError:
            if strict:
                raise
            fields = cast(Union[Unset, Fields], UNSET)

        def get_id() -> Union[Unset, str]:
            id = d.pop("id")
            return id

        try:
            id = get_id()
        except KeyError:
            if strict:
                raise
            id = cast(Union[Unset, str], UNSET)

        def get_project_id() -> Union[Unset, str]:
            project_id = d.pop("projectId")
            return project_id

        try:
            project_id = get_project_id()
        except KeyError:
            if strict:
                raise
            project_id = cast(Union[Unset, str], UNSET)

        def get_request_status() -> Union[Unset, RequestStatus]:
            request_status = UNSET
            _request_status = d.pop("requestStatus")
            if _request_status is not None and _request_status is not UNSET:
                try:
                    request_status = RequestStatus(_request_status)
                except ValueError:
                    request_status = RequestStatus.of_unknown(_request_status)

            return request_status

        try:
            request_status = get_request_status()
        except KeyError:
            if strict:
                raise
            request_status = cast(Union[Unset, RequestStatus], UNSET)

        def get_requestor() -> Union[Unset, RequestRequestor]:
            requestor: Union[Unset, Union[Unset, RequestRequestor]] = UNSET
            _requestor = d.pop("requestor")

            if not isinstance(_requestor, Unset):
                requestor = RequestRequestor.from_dict(_requestor)

            return requestor

        try:
            requestor = get_requestor()
        except KeyError:
            if strict:
                raise
            requestor = cast(Union[Unset, RequestRequestor], UNSET)

        def get_sample_groups() -> Union[Unset, List[RequestSampleGroup]]:
            sample_groups = []
            _sample_groups = d.pop("sampleGroups")
            for sample_groups_item_data in _sample_groups or []:
                sample_groups_item = RequestSampleGroup.from_dict(sample_groups_item_data, strict=False)

                sample_groups.append(sample_groups_item)

            return sample_groups

        try:
            sample_groups = get_sample_groups()
        except KeyError:
            if strict:
                raise
            sample_groups = cast(Union[Unset, List[RequestSampleGroup]], UNSET)

        def get_scheduled_on() -> Union[Unset, None, datetime.date]:
            scheduled_on: Union[Unset, None, datetime.date] = UNSET
            _scheduled_on = d.pop("scheduledOn")
            if _scheduled_on is not None and not isinstance(_scheduled_on, Unset):
                scheduled_on = isoparse(cast(str, _scheduled_on)).date()

            return scheduled_on

        try:
            scheduled_on = get_scheduled_on()
        except KeyError:
            if strict:
                raise
            scheduled_on = cast(Union[Unset, None, datetime.date], UNSET)

        def get_schema() -> Union[Unset, RequestSchemaProperty]:
            schema: Union[Unset, Union[Unset, RequestSchemaProperty]] = UNSET
            _schema = d.pop("schema")

            if not isinstance(_schema, Unset):
                schema = RequestSchemaProperty.from_dict(_schema)

            return schema

        try:
            schema = get_schema()
        except KeyError:
            if strict:
                raise
            schema = cast(Union[Unset, RequestSchemaProperty], UNSET)

        def get_tasks() -> Union[Unset, List[RequestTask]]:
            tasks = []
            _tasks = d.pop("tasks")
            for tasks_item_data in _tasks or []:
                tasks_item = RequestTask.from_dict(tasks_item_data, strict=False)

                tasks.append(tasks_item)

            return tasks

        try:
            tasks = get_tasks()
        except KeyError:
            if strict:
                raise
            tasks = cast(Union[Unset, List[RequestTask]], UNSET)

        def get_web_url() -> Union[Unset, str]:
            web_url = d.pop("webURL")
            return web_url

        try:
            web_url = get_web_url()
        except KeyError:
            if strict:
                raise
            web_url = cast(Union[Unset, str], UNSET)

        request = cls(
            api_url=api_url,
            assignees=assignees,
            created_at=created_at,
            creator=creator,
            display_id=display_id,
            fields=fields,
            id=id,
            project_id=project_id,
            request_status=request_status,
            requestor=requestor,
            sample_groups=sample_groups,
            scheduled_on=scheduled_on,
            schema=schema,
            tasks=tasks,
            web_url=web_url,
        )

        request.additional_properties = d
        return request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def api_url(self) -> str:
        """ The canonical url of the Request in the API. """
        if isinstance(self._api_url, Unset):
            raise NotPresentError(self, "api_url")
        return self._api_url

    @api_url.setter
    def api_url(self, value: str) -> None:
        self._api_url = value

    @api_url.deleter
    def api_url(self) -> None:
        self._api_url = UNSET

    @property
    def assignees(self) -> List[Union[RequestUserAssignee, RequestTeamAssignee, UnknownType]]:
        """ Array of assignees """
        if isinstance(self._assignees, Unset):
            raise NotPresentError(self, "assignees")
        return self._assignees

    @assignees.setter
    def assignees(self, value: List[Union[RequestUserAssignee, RequestTeamAssignee, UnknownType]]) -> None:
        self._assignees = value

    @assignees.deleter
    def assignees(self) -> None:
        self._assignees = UNSET

    @property
    def created_at(self) -> str:
        """ Date and time the request was created """
        if isinstance(self._created_at, Unset):
            raise NotPresentError(self, "created_at")
        return self._created_at

    @created_at.setter
    def created_at(self, value: str) -> None:
        self._created_at = value

    @created_at.deleter
    def created_at(self) -> None:
        self._created_at = UNSET

    @property
    def creator(self) -> RequestCreator:
        if isinstance(self._creator, Unset):
            raise NotPresentError(self, "creator")
        return self._creator

    @creator.setter
    def creator(self, value: RequestCreator) -> None:
        self._creator = value

    @creator.deleter
    def creator(self) -> None:
        self._creator = UNSET

    @property
    def display_id(self) -> str:
        """ User-friendly ID of the request """
        if isinstance(self._display_id, Unset):
            raise NotPresentError(self, "display_id")
        return self._display_id

    @display_id.setter
    def display_id(self, value: str) -> None:
        self._display_id = value

    @display_id.deleter
    def display_id(self) -> None:
        self._display_id = UNSET

    @property
    def fields(self) -> Fields:
        if isinstance(self._fields, Unset):
            raise NotPresentError(self, "fields")
        return self._fields

    @fields.setter
    def fields(self, value: Fields) -> None:
        self._fields = value

    @fields.deleter
    def fields(self) -> None:
        self._fields = UNSET

    @property
    def id(self) -> str:
        """ Unique ID for the request """
        if isinstance(self._id, Unset):
            raise NotPresentError(self, "id")
        return self._id

    @id.setter
    def id(self, value: str) -> None:
        self._id = value

    @id.deleter
    def id(self) -> None:
        self._id = UNSET

    @property
    def project_id(self) -> str:
        """ The ID of the project to which the request belongs. """
        if isinstance(self._project_id, Unset):
            raise NotPresentError(self, "project_id")
        return self._project_id

    @project_id.setter
    def project_id(self, value: str) -> None:
        self._project_id = value

    @project_id.deleter
    def project_id(self) -> None:
        self._project_id = UNSET

    @property
    def request_status(self) -> RequestStatus:
        if isinstance(self._request_status, Unset):
            raise NotPresentError(self, "request_status")
        return self._request_status

    @request_status.setter
    def request_status(self, value: RequestStatus) -> None:
        self._request_status = value

    @request_status.deleter
    def request_status(self) -> None:
        self._request_status = UNSET

    @property
    def requestor(self) -> RequestRequestor:
        if isinstance(self._requestor, Unset):
            raise NotPresentError(self, "requestor")
        return self._requestor

    @requestor.setter
    def requestor(self, value: RequestRequestor) -> None:
        self._requestor = value

    @requestor.deleter
    def requestor(self) -> None:
        self._requestor = UNSET

    @property
    def sample_groups(self) -> List[RequestSampleGroup]:
        if isinstance(self._sample_groups, Unset):
            raise NotPresentError(self, "sample_groups")
        return self._sample_groups

    @sample_groups.setter
    def sample_groups(self, value: List[RequestSampleGroup]) -> None:
        self._sample_groups = value

    @sample_groups.deleter
    def sample_groups(self) -> None:
        self._sample_groups = UNSET

    @property
    def scheduled_on(self) -> Optional[datetime.date]:
        """ Date the request is scheduled to be executed on, in YYYY-MM-DD format. """
        if isinstance(self._scheduled_on, Unset):
            raise NotPresentError(self, "scheduled_on")
        return self._scheduled_on

    @scheduled_on.setter
    def scheduled_on(self, value: Optional[datetime.date]) -> None:
        self._scheduled_on = value

    @scheduled_on.deleter
    def scheduled_on(self) -> None:
        self._scheduled_on = UNSET

    @property
    def schema(self) -> RequestSchemaProperty:
        if isinstance(self._schema, Unset):
            raise NotPresentError(self, "schema")
        return self._schema

    @schema.setter
    def schema(self, value: RequestSchemaProperty) -> None:
        self._schema = value

    @schema.deleter
    def schema(self) -> None:
        self._schema = UNSET

    @property
    def tasks(self) -> List[RequestTask]:
        if isinstance(self._tasks, Unset):
            raise NotPresentError(self, "tasks")
        return self._tasks

    @tasks.setter
    def tasks(self, value: List[RequestTask]) -> None:
        self._tasks = value

    @tasks.deleter
    def tasks(self) -> None:
        self._tasks = UNSET

    @property
    def web_url(self) -> str:
        """ URL of the request """
        if isinstance(self._web_url, Unset):
            raise NotPresentError(self, "web_url")
        return self._web_url

    @web_url.setter
    def web_url(self, value: str) -> None:
        self._web_url = value

    @web_url.deleter
    def web_url(self) -> None:
        self._web_url = UNSET



from typing import Any, cast, Dict, List, Type, TypeVar

import attr

from ..extensions import NotPresentError
from ..types import UNSET, Unset

T = TypeVar("T", bound="UnregisterEntities")


@attr.s(auto_attribs=True, repr=False)
class UnregisterEntities:
    """  """

    _entity_ids: List[str]
    _folder_id: str

    def __repr__(self):
        fields = []
        fields.append("entity_ids={}".format(repr(self._entity_ids)))
        fields.append("folder_id={}".format(repr(self._folder_id)))
        return "UnregisterEntities({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        entity_ids = self._entity_ids

        folder_id = self._folder_id

        field_dict: Dict[str, Any] = {}
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if entity_ids is not UNSET:
            field_dict["entityIds"] = entity_ids
        if folder_id is not UNSET:
            field_dict["folderId"] = folder_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_entity_ids() -> List[str]:
            entity_ids = cast(List[str], d.pop("entityIds"))

            return entity_ids

        try:
            entity_ids = get_entity_ids()
        except KeyError:
            if strict:
                raise
            entity_ids = cast(List[str], UNSET)

        def get_folder_id() -> str:
            folder_id = d.pop("folderId")
            return folder_id

        try:
            folder_id = get_folder_id()
        except KeyError:
            if strict:
                raise
            folder_id = cast(str, UNSET)

        unregister_entities = cls(
            entity_ids=entity_ids,
            folder_id=folder_id,
        )

        return unregister_entities

    @property
    def entity_ids(self) -> List[str]:
        """ Array of entity IDs """
        if isinstance(self._entity_ids, Unset):
            raise NotPresentError(self, "entity_ids")
        return self._entity_ids

    @entity_ids.setter
    def entity_ids(self, value: List[str]) -> None:
        self._entity_ids = value

    @property
    def folder_id(self) -> str:
        """ ID of the folder that the entities should be moved to """
        if isinstance(self._folder_id, Unset):
            raise NotPresentError(self, "folder_id")
        return self._folder_id

    @folder_id.setter
    def folder_id(self, value: str) -> None:
        self._folder_id = value



from typing import Any, cast, Dict, Type, TypeVar

import attr

from ..extensions import NotPresentError
from ..types import UNSET, Unset

T = TypeVar("T", bound="WarehouseCredentialsCreate")


@attr.s(auto_attribs=True, repr=False)
class WarehouseCredentialsCreate:
    """  """

    _expires_in: int

    def __repr__(self):
        fields = []
        fields.append("expires_in={}".format(repr(self._expires_in)))
        return "WarehouseCredentialsCreate({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        expires_in = self._expires_in

        field_dict: Dict[str, Any] = {}
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if expires_in is not UNSET:
            field_dict["expiresIn"] = expires_in

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_expires_in() -> int:
            expires_in = d.pop("expiresIn")
            return expires_in

        try:
            expires_in = get_expires_in()
        except KeyError:
            if strict:
                raise
            expires_in = cast(int, UNSET)

        warehouse_credentials_create = cls(
            expires_in=expires_in,
        )

        return warehouse_credentials_create

    @property
    def expires_in(self) -> int:
        """Duration, in seconds, that credentials should be active for. Must be greater than 0 and less than 3600."""
        if isinstance(self._expires_in, Unset):
            raise NotPresentError(self, "expires_in")
        return self._expires_in

    @expires_in.setter
    def expires_in(self, value: int) -> None:
        self._expires_in = value



from typing import Union

from ..extensions import UnknownType
from ..models.array_element_app_config_item import ArrayElementAppConfigItem
from ..models.boolean_app_config_item import BooleanAppConfigItem
from ..models.date_app_config_item import DateAppConfigItem
from ..models.datetime_app_config_item import DatetimeAppConfigItem
from ..models.entity_schema_app_config_item import EntitySchemaAppConfigItem
from ..models.field_app_config_item import FieldAppConfigItem
from ..models.float_app_config_item import FloatAppConfigItem
from ..models.generic_api_identified_app_config_item import GenericApiIdentifiedAppConfigItem
from ..models.integer_app_config_item import IntegerAppConfigItem
from ..models.json_app_config_item import JsonAppConfigItem
from ..models.secure_text_app_config_item import SecureTextAppConfigItem
from ..models.text_app_config_item import TextAppConfigItem

AppConfigItem = Union[
    ArrayElementAppConfigItem,
    DateAppConfigItem,
    DatetimeAppConfigItem,
    JsonAppConfigItem,
    EntitySchemaAppConfigItem,
    FieldAppConfigItem,
    BooleanAppConfigItem,
    IntegerAppConfigItem,
    FloatAppConfigItem,
    TextAppConfigItem,
    GenericApiIdentifiedAppConfigItem,
    SecureTextAppConfigItem,
    UnknownType,
]



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.benchling_app_manifest_alpha_settings_lifecycle_management import (
    BenchlingAppManifestAlphaSettingsLifecycleManagement,
)
from ..types import UNSET, Unset

T = TypeVar("T", bound="BenchlingAppManifestAlphaSettings")


@attr.s(auto_attribs=True, repr=False)
class BenchlingAppManifestAlphaSettings:
    """  """

    _canvas_webhook_url: Union[Unset, str] = UNSET
    _lifecycle_management: Union[Unset, BenchlingAppManifestAlphaSettingsLifecycleManagement] = UNSET
    _lifecycle_webhook_url: Union[Unset, str] = UNSET
    _webhook_url: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("canvas_webhook_url={}".format(repr(self._canvas_webhook_url)))
        fields.append("lifecycle_management={}".format(repr(self._lifecycle_management)))
        fields.append("lifecycle_webhook_url={}".format(repr(self._lifecycle_webhook_url)))
        fields.append("webhook_url={}".format(repr(self._webhook_url)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "BenchlingAppManifestAlphaSettings({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        canvas_webhook_url = self._canvas_webhook_url
        lifecycle_management: Union[Unset, int] = UNSET
        if not isinstance(self._lifecycle_management, Unset):
            lifecycle_management = self._lifecycle_management.value

        lifecycle_webhook_url = self._lifecycle_webhook_url
        webhook_url = self._webhook_url

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if canvas_webhook_url is not UNSET:
            field_dict["canvasWebhookUrl"] = canvas_webhook_url
        if lifecycle_management is not UNSET:
            field_dict["lifecycleManagement"] = lifecycle_management
        if lifecycle_webhook_url is not UNSET:
            field_dict["lifecycleWebhookUrl"] = lifecycle_webhook_url
        if webhook_url is not UNSET:
            field_dict["webhookUrl"] = webhook_url

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_canvas_webhook_url() -> Union[Unset, str]:
            canvas_webhook_url = d.pop("canvasWebhookUrl")
            return canvas_webhook_url

        try:
            canvas_webhook_url = get_canvas_webhook_url()
        except KeyError:
            if strict:
                raise
            canvas_webhook_url = cast(Union[Unset, str], UNSET)

        def get_lifecycle_management() -> Union[Unset, BenchlingAppManifestAlphaSettingsLifecycleManagement]:
            lifecycle_management = UNSET
            _lifecycle_management = d.pop("lifecycleManagement")
            if _lifecycle_management is not None and _lifecycle_management is not UNSET:
                try:
                    lifecycle_management = BenchlingAppManifestAlphaSettingsLifecycleManagement(
                        _lifecycle_management
                    )
                except ValueError:
                    lifecycle_management = BenchlingAppManifestAlphaSettingsLifecycleManagement.of_unknown(
                        _lifecycle_management
                    )

            return lifecycle_management

        try:
            lifecycle_management = get_lifecycle_management()
        except KeyError:
            if strict:
                raise
            lifecycle_management = cast(
                Union[Unset, BenchlingAppManifestAlphaSettingsLifecycleManagement], UNSET
            )

        def get_lifecycle_webhook_url() -> Union[Unset, str]:
            lifecycle_webhook_url = d.pop("lifecycleWebhookUrl")
            return lifecycle_webhook_url

        try:
            lifecycle_webhook_url = get_lifecycle_webhook_url()
        except KeyError:
            if strict:
                raise
            lifecycle_webhook_url = cast(Union[Unset, str], UNSET)

        def get_webhook_url() -> Union[Unset, str]:
            webhook_url = d.pop("webhookUrl")
            return webhook_url

        try:
            webhook_url = get_webhook_url()
        except KeyError:
            if strict:
                raise
            webhook_url = cast(Union[Unset, str], UNSET)

        benchling_app_manifest_alpha_settings = cls(
            canvas_webhook_url=canvas_webhook_url,
            lifecycle_management=lifecycle_management,
            lifecycle_webhook_url=lifecycle_webhook_url,
            webhook_url=webhook_url,
        )

        benchling_app_manifest_alpha_settings.additional_properties = d
        return benchling_app_manifest_alpha_settings

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def canvas_webhook_url(self) -> str:
        """URL that Benchling sends app canvas payloads to. It will override the default webhookUrl This destination should be backed by an endpoint owned by your application."""
        if isinstance(self._canvas_webhook_url, Unset):
            raise NotPresentError(self, "canvas_webhook_url")
        return self._canvas_webhook_url

    @canvas_webhook_url.setter
    def canvas_webhook_url(self, value: str) -> None:
        self._canvas_webhook_url = value

    @canvas_webhook_url.deleter
    def canvas_webhook_url(self) -> None:
        self._canvas_webhook_url = UNSET

    @property
    def lifecycle_management(self) -> BenchlingAppManifestAlphaSettingsLifecycleManagement:
        """Specify whether app will be activated automatically or manually. If automatic is specified, a webhook url is required."""
        if isinstance(self._lifecycle_management, Unset):
            raise NotPresentError(self, "lifecycle_management")
        return self._lifecycle_management

    @lifecycle_management.setter
    def lifecycle_management(self, value: BenchlingAppManifestAlphaSettingsLifecycleManagement) -> None:
        self._lifecycle_management = value

    @lifecycle_management.deleter
    def lifecycle_management(self) -> None:
        self._lifecycle_management = UNSET

    @property
    def lifecycle_webhook_url(self) -> str:
        """URL that Benchling sends app lifecycle payloads to. It will override the default webhookUrl This destination should be backed by an endpoint owned by your application."""
        if isinstance(self._lifecycle_webhook_url, Unset):
            raise NotPresentError(self, "lifecycle_webhook_url")
        return self._lifecycle_webhook_url

    @lifecycle_webhook_url.setter
    def lifecycle_webhook_url(self, value: str) -> None:
        self._lifecycle_webhook_url = value

    @lifecycle_webhook_url.deleter
    def lifecycle_webhook_url(self) -> None:
        self._lifecycle_webhook_url = UNSET

    @property
    def webhook_url(self) -> str:
        """URL that Benchling sends app interaction payloads to. This is the default URL if no other webhook URLs are specified in the app's manifest. This destination should be backed by an endpoint owned by your application."""
        if isinstance(self._webhook_url, Unset):
            raise NotPresentError(self, "webhook_url")
        return self._webhook_url

    @webhook_url.setter
    def webhook_url(self, value: str) -> None:
        self._webhook_url = value

    @webhook_url.deleter
    def webhook_url(self) -> None:
        self._webhook_url = UNSET



from typing import Any, Dict, Optional, Union

import httpx

from ...client import Client
from ...models.async_task_link import AsyncTaskLink
from ...models.bad_request_error import BadRequestError
from ...models.molecule_bulk_upsert_request import MoleculeBulkUpsertRequest
from ...types import Response, UNSET, Unset


def _get_kwargs(
    *,
    client: Client,
    json_body: MoleculeBulkUpsertRequest,
    returning: Union[Unset, str] = UNSET,
) -> Dict[str, Any]:
    url = "{}/molecules:bulk-upsert".format(client.base_url)

    headers: Dict[str, Any] = client.httpx_client.headers
    headers.update(client.get_headers())

    cookies: Dict[str, Any] = client.httpx_client.cookies
    cookies.update(client.get_cookies())

    params: Dict[str, Any] = {}
    if not isinstance(returning, Unset) and returning is not None:
        params["returning"] = returning

    json_json_body = json_body.to_dict()

    return {
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "json": json_json_body,
        "params": params,
    }


def _parse_response(*, response: httpx.Response) -> Optional[Union[AsyncTaskLink, BadRequestError]]:
    if response.status_code == 202:
        response_202 = AsyncTaskLink.from_dict(response.json(), strict=False)

        return response_202
    if response.status_code == 400:
        response_400 = BadRequestError.from_dict(response.json(), strict=False)

        return response_400
    return None


def _build_response(*, response: httpx.Response) -> Response[Union[AsyncTaskLink, BadRequestError]]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    *,
    client: Client,
    json_body: MoleculeBulkUpsertRequest,
    returning: Union[Unset, str] = UNSET,
) -> Response[Union[AsyncTaskLink, BadRequestError]]:
    kwargs = _get_kwargs(
        client=client,
        json_body=json_body,
        returning=returning,
    )

    response = client.httpx_client.post(
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    *,
    client: Client,
    json_body: MoleculeBulkUpsertRequest,
    returning: Union[Unset, str] = UNSET,
) -> Optional[Union[AsyncTaskLink, BadRequestError]]:
    """All entities and their schemas must be within the same registry.

    This operation performs the following actions:
    1. Any existing objects are looked up in Benchling by the provided entity registry ID.
    2. Then, all objects are either created or updated accordingly, temporarily skipping any schema field links between objects.
    3. Schema field links can be populated using entity registry IDs or API IDs. In the `value` field of the [Field](#/components/schemas/FieldWithResolution) resource, the object `{\"entityRegistryId\": ENTITY_REGISTRY_ID}` may be provided instead of the API ID if desired (see example value). You may link to objects being created in the same operation.
    4. Entities are registered, using the provided name and entity registry ID.

    If any action fails, the whole operation is canceled and no objects are created or updated.
    """

    return sync_detailed(
        client=client,
        json_body=json_body,
        returning=returning,
    ).parsed


async def asyncio_detailed(
    *,
    client: Client,
    json_body: MoleculeBulkUpsertRequest,
    returning: Union[Unset, str] = UNSET,
) -> Response[Union[AsyncTaskLink, BadRequestError]]:
    kwargs = _get_kwargs(
        client=client,
        json_body=json_body,
        returning=returning,
    )

    async with httpx.AsyncClient() as _client:
        response = await _client.post(**kwargs)

    return _build_response(response=response)


async def asyncio(
    *,
    client: Client,
    json_body: MoleculeBulkUpsertRequest,
    returning: Union[Unset, str] = UNSET,
) -> Optional[Union[AsyncTaskLink, BadRequestError]]:
    """All entities and their schemas must be within the same registry.

    This operation performs the following actions:
    1. Any existing objects are looked up in Benchling by the provided entity registry ID.
    2. Then, all objects are either created or updated accordingly, temporarily skipping any schema field links between objects.
    3. Schema field links can be populated using entity registry IDs or API IDs. In the `value` field of the [Field](#/components/schemas/FieldWithResolution) resource, the object `{\"entityRegistryId\": ENTITY_REGISTRY_ID}` may be provided instead of the API ID if desired (see example value). You may link to objects being created in the same operation.
    4. Entities are registered, using the provided name and entity registry ID.

    If any action fails, the whole operation is canceled and no objects are created or updated.
    """

    return (
        await asyncio_detailed(
            client=client,
            json_body=json_body,
            returning=returning,
        )
    ).parsed



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.rna_sequence import RnaSequence
from ..types import UNSET, Unset

T = TypeVar("T", bound="RnaSequencesPaginatedList")


@attr.s(auto_attribs=True, repr=False)
class RnaSequencesPaginatedList:
    """  """

    _next_token: Union[Unset, str] = UNSET
    _rna_sequences: Union[Unset, List[RnaSequence]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("next_token={}".format(repr(self._next_token)))
        fields.append("rna_sequences={}".format(repr(self._rna_sequences)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "RnaSequencesPaginatedList({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        next_token = self._next_token
        rna_sequences: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._rna_sequences, Unset):
            rna_sequences = []
            for rna_sequences_item_data in self._rna_sequences:
                rna_sequences_item = rna_sequences_item_data.to_dict()

                rna_sequences.append(rna_sequences_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if next_token is not UNSET:
            field_dict["nextToken"] = next_token
        if rna_sequences is not UNSET:
            field_dict["rnaSequences"] = rna_sequences

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_next_token() -> Union[Unset, str]:
            next_token = d.pop("nextToken")
            return next_token

        try:
            next_token = get_next_token()
        except KeyError:
            if strict:
                raise
            next_token = cast(Union[Unset, str], UNSET)

        def get_rna_sequences() -> Union[Unset, List[RnaSequence]]:
            rna_sequences = []
            _rna_sequences = d.pop("rnaSequences")
            for rna_sequences_item_data in _rna_sequences or []:
                rna_sequences_item = RnaSequence.from_dict(rna_sequences_item_data, strict=False)

                rna_sequences.append(rna_sequences_item)

            return rna_sequences

        try:
            rna_sequences = get_rna_sequences()
        except KeyError:
            if strict:
                raise
            rna_sequences = cast(Union[Unset, List[RnaSequence]], UNSET)

        rna_sequences_paginated_list = cls(
            next_token=next_token,
            rna_sequences=rna_sequences,
        )

        rna_sequences_paginated_list.additional_properties = d
        return rna_sequences_paginated_list

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def next_token(self) -> str:
        if isinstance(self._next_token, Unset):
            raise NotPresentError(self, "next_token")
        return self._next_token

    @next_token.setter
    def next_token(self, value: str) -> None:
        self._next_token = value

    @next_token.deleter
    def next_token(self) -> None:
        self._next_token = UNSET

    @property
    def rna_sequences(self) -> List[RnaSequence]:
        if isinstance(self._rna_sequences, Unset):
            raise NotPresentError(self, "rna_sequences")
        return self._rna_sequences

    @rna_sequences.setter
    def rna_sequences(self, value: List[RnaSequence]) -> None:
        self._rna_sequences = value

    @rna_sequences.deleter
    def rna_sequences(self) -> None:
        self._rna_sequences = UNSET



from typing import Any, cast, Dict, List, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.session_message_create import SessionMessageCreate
from ..models.session_update_status import SessionUpdateStatus
from ..types import UNSET, Unset

T = TypeVar("T", bound="SessionUpdate")


@attr.s(auto_attribs=True, repr=False)
class SessionUpdate:
    """ Update a session's messages or increase timeoutSeconds. """

    _messages: Union[Unset, List[SessionMessageCreate]] = UNSET
    _status: Union[Unset, SessionUpdateStatus] = UNSET
    _timeout_seconds: Union[Unset, int] = UNSET

    def __repr__(self):
        fields = []
        fields.append("messages={}".format(repr(self._messages)))
        fields.append("status={}".format(repr(self._status)))
        fields.append("timeout_seconds={}".format(repr(self._timeout_seconds)))
        return "SessionUpdate({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        messages: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._messages, Unset):
            messages = []
            for messages_item_data in self._messages:
                messages_item = messages_item_data.to_dict()

                messages.append(messages_item)

        status: Union[Unset, int] = UNSET
        if not isinstance(self._status, Unset):
            status = self._status.value

        timeout_seconds = self._timeout_seconds

        field_dict: Dict[str, Any] = {}
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if messages is not UNSET:
            field_dict["messages"] = messages
        if status is not UNSET:
            field_dict["status"] = status
        if timeout_seconds is not UNSET:
            field_dict["timeoutSeconds"] = timeout_seconds

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_messages() -> Union[Unset, List[SessionMessageCreate]]:
            messages = []
            _messages = d.pop("messages")
            for messages_item_data in _messages or []:
                messages_item = SessionMessageCreate.from_dict(messages_item_data, strict=False)

                messages.append(messages_item)

            return messages

        try:
            messages = get_messages()
        except KeyError:
            if strict:
                raise
            messages = cast(Union[Unset, List[SessionMessageCreate]], UNSET)

        def get_status() -> Union[Unset, SessionUpdateStatus]:
            status = UNSET
            _status = d.pop("status")
            if _status is not None and _status is not UNSET:
                try:
                    status = SessionUpdateStatus(_status)
                except ValueError:
                    status = SessionUpdateStatus.of_unknown(_status)

            return status

        try:
            status = get_status()
        except KeyError:
            if strict:
                raise
            status = cast(Union[Unset, SessionUpdateStatus], UNSET)

        def get_timeout_seconds() -> Union[Unset, int]:
            timeout_seconds = d.pop("timeoutSeconds")
            return timeout_seconds

        try:
            timeout_seconds = get_timeout_seconds()
        except KeyError:
            if strict:
                raise
            timeout_seconds = cast(Union[Unset, int], UNSET)

        session_update = cls(
            messages=messages,
            status=status,
            timeout_seconds=timeout_seconds,
        )

        return session_update

    @property
    def messages(self) -> List[SessionMessageCreate]:
        """An array of `SessionMessage` describing the current session state."""
        if isinstance(self._messages, Unset):
            raise NotPresentError(self, "messages")
        return self._messages

    @messages.setter
    def messages(self, value: List[SessionMessageCreate]) -> None:
        self._messages = value

    @messages.deleter
    def messages(self) -> None:
        self._messages = UNSET

    @property
    def status(self) -> SessionUpdateStatus:
        """ Values that can be specified when updating the status of a Session """
        if isinstance(self._status, Unset):
            raise NotPresentError(self, "status")
        return self._status

    @status.setter
    def status(self, value: SessionUpdateStatus) -> None:
        self._status = value

    @status.deleter
    def status(self) -> None:
        self._status = UNSET

    @property
    def timeout_seconds(self) -> int:
        """Timeout in seconds, a value between 1 second and 30 days. Once set, it can only be increased, not decreased."""
        if isinstance(self._timeout_seconds, Unset):
            raise NotPresentError(self, "timeout_seconds")
        return self._timeout_seconds

    @timeout_seconds.setter
    def timeout_seconds(self, value: int) -> None:
        self._timeout_seconds = value

    @timeout_seconds.deleter
    def timeout_seconds(self) -> None:
        self._timeout_seconds = UNSET



from typing import Union

from ..extensions import UnknownType
from ..models.aa_sequence import AaSequence
from ..models.custom_entity import CustomEntity
from ..models.dna_oligo import DnaOligo
from ..models.dna_sequence import DnaSequence
from ..models.mixture import Mixture
from ..models.rna_oligo import RnaOligo

Entity = Union[DnaSequence, AaSequence, Mixture, DnaOligo, RnaOligo, CustomEntity, UnknownType]



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.container import Container
from ..types import UNSET, Unset

T = TypeVar("T", bound="TransfersAsyncTaskResponse")


@attr.s(auto_attribs=True, repr=False)
class TransfersAsyncTaskResponse:
    """  """

    _destination_containers: Union[Unset, List[Container]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("destination_containers={}".format(repr(self._destination_containers)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "TransfersAsyncTaskResponse({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        destination_containers: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._destination_containers, Unset):
            destination_containers = []
            for destination_containers_item_data in self._destination_containers:
                destination_containers_item = destination_containers_item_data.to_dict()

                destination_containers.append(destination_containers_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if destination_containers is not UNSET:
            field_dict["destinationContainers"] = destination_containers

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_destination_containers() -> Union[Unset, List[Container]]:
            destination_containers = []
            _destination_containers = d.pop("destinationContainers")
            for destination_containers_item_data in _destination_containers or []:
                destination_containers_item = Container.from_dict(
                    destination_containers_item_data, strict=False
                )

                destination_containers.append(destination_containers_item)

            return destination_containers

        try:
            destination_containers = get_destination_containers()
        except KeyError:
            if strict:
                raise
            destination_containers = cast(Union[Unset, List[Container]], UNSET)

        transfers_async_task_response = cls(
            destination_containers=destination_containers,
        )

        transfers_async_task_response.additional_properties = d
        return transfers_async_task_response

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def destination_containers(self) -> List[Container]:
        if isinstance(self._destination_containers, Unset):
            raise NotPresentError(self, "destination_containers")
        return self._destination_containers

    @destination_containers.setter
    def destination_containers(self, value: List[Container]) -> None:
        self._destination_containers = value

    @destination_containers.deleter
    def destination_containers(self) -> None:
        self._destination_containers = UNSET



import benchling_api_client.v2.extensions

NotPresentError = benchling_api_client.v2.extensions.NotPresentError
UnknownType = benchling_api_client.v2.extensions.UnknownType
T = benchling_api_client.v2.extensions.T
Enums = benchling_api_client.v2.extensions.Enums



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.container_schema import ContainerSchema
from ..types import UNSET, Unset

T = TypeVar("T", bound="ContainerSchemasList")


@attr.s(auto_attribs=True, repr=False)
class ContainerSchemasList:
    """  """

    _container_schemas: Union[Unset, List[ContainerSchema]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("container_schemas={}".format(repr(self._container_schemas)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "ContainerSchemasList({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        container_schemas: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._container_schemas, Unset):
            container_schemas = []
            for container_schemas_item_data in self._container_schemas:
                container_schemas_item = container_schemas_item_data.to_dict()

                container_schemas.append(container_schemas_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if container_schemas is not UNSET:
            field_dict["containerSchemas"] = container_schemas

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_container_schemas() -> Union[Unset, List[ContainerSchema]]:
            container_schemas = []
            _container_schemas = d.pop("containerSchemas")
            for container_schemas_item_data in _container_schemas or []:
                container_schemas_item = ContainerSchema.from_dict(container_schemas_item_data, strict=False)

                container_schemas.append(container_schemas_item)

            return container_schemas

        try:
            container_schemas = get_container_schemas()
        except KeyError:
            if strict:
                raise
            container_schemas = cast(Union[Unset, List[ContainerSchema]], UNSET)

        container_schemas_list = cls(
            container_schemas=container_schemas,
        )

        container_schemas_list.additional_properties = d
        return container_schemas_list

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def container_schemas(self) -> List[ContainerSchema]:
        if isinstance(self._container_schemas, Unset):
            raise NotPresentError(self, "container_schemas")
        return self._container_schemas

    @container_schemas.setter
    def container_schemas(self, value: List[ContainerSchema]) -> None:
        self._container_schemas = value

    @container_schemas.deleter
    def container_schemas(self) -> None:
        self._container_schemas = UNSET



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.entry import Entry
from ..types import UNSET, Unset

T = TypeVar("T", bound="EntryById")


@attr.s(auto_attribs=True, repr=False)
class EntryById:
    """  """

    _entry: Union[Unset, Entry] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("entry={}".format(repr(self._entry)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "EntryById({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        entry: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self._entry, Unset):
            entry = self._entry.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if entry is not UNSET:
            field_dict["entry"] = entry

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_entry() -> Union[Unset, Entry]:
            entry: Union[Unset, Union[Unset, Entry]] = UNSET
            _entry = d.pop("entry")

            if not isinstance(_entry, Unset):
                entry = Entry.from_dict(_entry)

            return entry

        try:
            entry = get_entry()
        except KeyError:
            if strict:
                raise
            entry = cast(Union[Unset, Entry], UNSET)

        entry_by_id = cls(
            entry=entry,
        )

        entry_by_id.additional_properties = d
        return entry_by_id

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def entry(self) -> Entry:
        """Entries are notes that users can take. They're organized by "days" (which are user-configurable) and modeled within each day as a list of "notes." Each note has a type - the simplest is a "text" type, but lists, tables, and external files are also supported.

        *Note:* the current Entry resource has a few limitations:
        - Formatting information is not yet supported. Header formatting, bolding, and other stylistic information is not presented.
        - Data in tables is presented as text always - numeric values will need to be parsed into floats or integers, as appropriate.

        Note: Data in Results tables are not accessible through this API call. Results table data can be called through the Results API calls.
        """
        if isinstance(self._entry, Unset):
            raise NotPresentError(self, "entry")
        return self._entry

    @entry.setter
    def entry(self, value: Entry) -> None:
        self._entry = value

    @entry.deleter
    def entry(self) -> None:
        self._entry = UNSET



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.organization import Organization
from ..types import UNSET, Unset

T = TypeVar("T", bound="OrganizationsPaginatedList")


@attr.s(auto_attribs=True, repr=False)
class OrganizationsPaginatedList:
    """  """

    _organizations: Union[Unset, List[Organization]] = UNSET
    _next_token: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("organizations={}".format(repr(self._organizations)))
        fields.append("next_token={}".format(repr(self._next_token)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "OrganizationsPaginatedList({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        organizations: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._organizations, Unset):
            organizations = []
            for organizations_item_data in self._organizations:
                organizations_item = organizations_item_data.to_dict()

                organizations.append(organizations_item)

        next_token = self._next_token

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if organizations is not UNSET:
            field_dict["organizations"] = organizations
        if next_token is not UNSET:
            field_dict["nextToken"] = next_token

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_organizations() -> Union[Unset, List[Organization]]:
            organizations = []
            _organizations = d.pop("organizations")
            for organizations_item_data in _organizations or []:
                organizations_item = Organization.from_dict(organizations_item_data, strict=False)

                organizations.append(organizations_item)

            return organizations

        try:
            organizations = get_organizations()
        except KeyError:
            if strict:
                raise
            organizations = cast(Union[Unset, List[Organization]], UNSET)

        def get_next_token() -> Union[Unset, str]:
            next_token = d.pop("nextToken")
            return next_token

        try:
            next_token = get_next_token()
        except KeyError:
            if strict:
                raise
            next_token = cast(Union[Unset, str], UNSET)

        organizations_paginated_list = cls(
            organizations=organizations,
            next_token=next_token,
        )

        organizations_paginated_list.additional_properties = d
        return organizations_paginated_list

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def organizations(self) -> List[Organization]:
        if isinstance(self._organizations, Unset):
            raise NotPresentError(self, "organizations")
        return self._organizations

    @organizations.setter
    def organizations(self, value: List[Organization]) -> None:
        self._organizations = value

    @organizations.deleter
    def organizations(self) -> None:
        self._organizations = UNSET

    @property
    def next_token(self) -> str:
        if isinstance(self._next_token, Unset):
            raise NotPresentError(self, "next_token")
        return self._next_token

    @next_token.setter
    def next_token(self, value: str) -> None:
        self._next_token = value

    @next_token.deleter
    def next_token(self) -> None:
        self._next_token = UNSET



from typing import Any, cast, Dict, List, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError, UnknownType
from ..models.member_collaborator import MemberCollaborator
from ..models.principal_collaborator import PrincipalCollaborator
from ..models.worklist_type import WorklistType
from ..types import UNSET, Unset

T = TypeVar("T", bound="WorklistCreate")


@attr.s(auto_attribs=True, repr=False)
class WorklistCreate:
    """  """

    _name: str
    _type: WorklistType
    _collaborations: Union[Unset, List[Union[PrincipalCollaborator, MemberCollaborator, UnknownType]]] = UNSET
    _worklist_item_ids: Union[Unset, List[str]] = UNSET

    def __repr__(self):
        fields = []
        fields.append("name={}".format(repr(self._name)))
        fields.append("type={}".format(repr(self._type)))
        fields.append("collaborations={}".format(repr(self._collaborations)))
        fields.append("worklist_item_ids={}".format(repr(self._worklist_item_ids)))
        return "WorklistCreate({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        name = self._name
        type = self._type.value

        collaborations: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._collaborations, Unset):
            collaborations = []
            for collaborations_item_data in self._collaborations:
                if isinstance(collaborations_item_data, UnknownType):
                    collaborations_item = collaborations_item_data.value
                elif isinstance(collaborations_item_data, PrincipalCollaborator):
                    collaborations_item = collaborations_item_data.to_dict()

                else:
                    collaborations_item = collaborations_item_data.to_dict()

                collaborations.append(collaborations_item)

        worklist_item_ids: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._worklist_item_ids, Unset):
            worklist_item_ids = self._worklist_item_ids

        field_dict: Dict[str, Any] = {}
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if name is not UNSET:
            field_dict["name"] = name
        if type is not UNSET:
            field_dict["type"] = type
        if collaborations is not UNSET:
            field_dict["collaborations"] = collaborations
        if worklist_item_ids is not UNSET:
            field_dict["worklistItemIds"] = worklist_item_ids

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_name() -> str:
            name = d.pop("name")
            return name

        try:
            name = get_name()
        except KeyError:
            if strict:
                raise
            name = cast(str, UNSET)

        def get_type() -> WorklistType:
            _type = d.pop("type")
            try:
                type = WorklistType(_type)
            except ValueError:
                type = WorklistType.of_unknown(_type)

            return type

        try:
            type = get_type()
        except KeyError:
            if strict:
                raise
            type = cast(WorklistType, UNSET)

        def get_collaborations() -> Union[
            Unset, List[Union[PrincipalCollaborator, MemberCollaborator, UnknownType]]
        ]:
            collaborations = []
            _collaborations = d.pop("collaborations")
            for collaborations_item_data in _collaborations or []:

                def _parse_collaborations_item(
                    data: Union[Dict[str, Any]]
                ) -> Union[PrincipalCollaborator, MemberCollaborator, UnknownType]:
                    collaborations_item: Union[PrincipalCollaborator, MemberCollaborator, UnknownType]
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        collaboration_create = PrincipalCollaborator.from_dict(data, strict=True)

                        return collaboration_create
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        collaboration_create = MemberCollaborator.from_dict(data, strict=True)

                        return collaboration_create
                    except:  # noqa: E722
                        pass
                    return UnknownType(data)

                collaborations_item = _parse_collaborations_item(collaborations_item_data)

                collaborations.append(collaborations_item)

            return collaborations

        try:
            collaborations = get_collaborations()
        except KeyError:
            if strict:
                raise
            collaborations = cast(
                Union[Unset, List[Union[PrincipalCollaborator, MemberCollaborator, UnknownType]]], UNSET
            )

        def get_worklist_item_ids() -> Union[Unset, List[str]]:
            worklist_item_ids = cast(List[str], d.pop("worklistItemIds"))

            return worklist_item_ids

        try:
            worklist_item_ids = get_worklist_item_ids()
        except KeyError:
            if strict:
                raise
            worklist_item_ids = cast(Union[Unset, List[str]], UNSET)

        worklist_create = cls(
            name=name,
            type=type,
            collaborations=collaborations,
            worklist_item_ids=worklist_item_ids,
        )

        return worklist_create

    @property
    def name(self) -> str:
        """ Name of the worklist """
        if isinstance(self._name, Unset):
            raise NotPresentError(self, "name")
        return self._name

    @name.setter
    def name(self, value: str) -> None:
        self._name = value

    @property
    def type(self) -> WorklistType:
        """The type of items a worklist contains."""
        if isinstance(self._type, Unset):
            raise NotPresentError(self, "type")
        return self._type

    @type.setter
    def type(self, value: WorklistType) -> None:
        self._type = value

    @property
    def collaborations(self) -> List[Union[PrincipalCollaborator, MemberCollaborator, UnknownType]]:
        if isinstance(self._collaborations, Unset):
            raise NotPresentError(self, "collaborations")
        return self._collaborations

    @collaborations.setter
    def collaborations(
        self, value: List[Union[PrincipalCollaborator, MemberCollaborator, UnknownType]]
    ) -> None:
        self._collaborations = value

    @collaborations.deleter
    def collaborations(self) -> None:
        self._collaborations = UNSET

    @property
    def worklist_item_ids(self) -> List[str]:
        """An ordered set of IDs to assign as worklist items. IDs should reference existing items which fit the worklist's specific type. For instance, a worklist of type container should only have item IDs which represent containers."""
        if isinstance(self._worklist_item_ids, Unset):
            raise NotPresentError(self, "worklist_item_ids")
        return self._worklist_item_ids

    @worklist_item_ids.setter
    def worklist_item_ids(self, value: List[str]) -> None:
        self._worklist_item_ids = value

    @worklist_item_ids.deleter
    def worklist_item_ids(self) -> None:
        self._worklist_item_ids = UNSET



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError, UnknownType
from ..models.aa_sequence import AaSequence
from ..models.batch import Batch
from ..models.custom_entity import CustomEntity
from ..models.dna_oligo import DnaOligo
from ..models.dna_sequence import DnaSequence
from ..models.inaccessible_resource import InaccessibleResource
from ..models.measurement import Measurement
from ..models.mixture import Mixture
from ..models.rna_oligo import RnaOligo
from ..types import UNSET, Unset

T = TypeVar("T", bound="ContainerContent")


@attr.s(auto_attribs=True, repr=False)
class ContainerContent:
    """  """

    _batch: Union[Unset, None, Batch, InaccessibleResource, UnknownType] = UNSET
    _concentration: Union[Unset, Measurement] = UNSET
    _entity: Union[
        Unset,
        None,
        Union[DnaSequence, AaSequence, Mixture, DnaOligo, RnaOligo, CustomEntity, UnknownType],
        InaccessibleResource,
        UnknownType,
    ] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("batch={}".format(repr(self._batch)))
        fields.append("concentration={}".format(repr(self._concentration)))
        fields.append("entity={}".format(repr(self._entity)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "ContainerContent({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        batch: Union[Unset, None, Dict[str, Any]]
        if isinstance(self._batch, Unset):
            batch = UNSET
        elif self._batch is None:
            batch = None
        elif isinstance(self._batch, UnknownType):
            batch = self._batch.value
        elif isinstance(self._batch, Batch):
            batch = self._batch.to_dict()

        else:
            batch = self._batch.to_dict()

        concentration: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self._concentration, Unset):
            concentration = self._concentration.to_dict()

        entity: Union[Unset, None, Union[Dict[str, Any]], Dict[str, Any]]
        if isinstance(self._entity, Unset):
            entity = UNSET
        elif self._entity is None:
            entity = None
        elif isinstance(self._entity, UnknownType):
            entity = self._entity.value
        elif isinstance(self._entity, object):
            if isinstance(self._entity, UnknownType):
                entity = self._entity.value
            elif isinstance(self._entity, DnaSequence):
                entity = self._entity.to_dict()

            elif isinstance(self._entity, AaSequence):
                entity = self._entity.to_dict()

            elif isinstance(self._entity, Mixture):
                entity = self._entity.to_dict()

            elif isinstance(self._entity, DnaOligo):
                entity = self._entity.to_dict()

            elif isinstance(self._entity, RnaOligo):
                entity = self._entity.to_dict()

            else:
                entity = self._entity.to_dict()

        else:
            entity = self._entity.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if batch is not UNSET:
            field_dict["batch"] = batch
        if concentration is not UNSET:
            field_dict["concentration"] = concentration
        if entity is not UNSET:
            field_dict["entity"] = entity

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_batch() -> Union[Unset, None, Batch, InaccessibleResource, UnknownType]:
            def _parse_batch(
                data: Union[Unset, None, Dict[str, Any]]
            ) -> Union[Unset, None, Batch, InaccessibleResource, UnknownType]:
                batch: Union[Unset, None, Batch, InaccessibleResource, UnknownType]
                if data is None:
                    return data
                if isinstance(data, Unset):
                    return data
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    batch_or_inaccessible_resource = Batch.from_dict(data, strict=True)

                    return batch_or_inaccessible_resource
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    batch_or_inaccessible_resource = InaccessibleResource.from_dict(data, strict=True)

                    return batch_or_inaccessible_resource
                except:  # noqa: E722
                    pass
                return UnknownType(data)

            batch = _parse_batch(d.pop("batch"))

            return batch

        try:
            batch = get_batch()
        except KeyError:
            if strict:
                raise
            batch = cast(Union[Unset, None, Batch, InaccessibleResource, UnknownType], UNSET)

        def get_concentration() -> Union[Unset, Measurement]:
            concentration: Union[Unset, Union[Unset, Measurement]] = UNSET
            _concentration = d.pop("concentration")

            if not isinstance(_concentration, Unset):
                concentration = Measurement.from_dict(_concentration)

            return concentration

        try:
            concentration = get_concentration()
        except KeyError:
            if strict:
                raise
            concentration = cast(Union[Unset, Measurement], UNSET)

        def get_entity() -> Union[
            Unset,
            None,
            Union[DnaSequence, AaSequence, Mixture, DnaOligo, RnaOligo, CustomEntity, UnknownType],
            InaccessibleResource,
            UnknownType,
        ]:
            def _parse_entity(
                data: Union[Unset, None, Union[Dict[str, Any]], Dict[str, Any]]
            ) -> Union[
                Unset,
                None,
                Union[DnaSequence, AaSequence, Mixture, DnaOligo, RnaOligo, CustomEntity, UnknownType],
                InaccessibleResource,
                UnknownType,
            ]:
                entity: Union[
                    Unset,
                    None,
                    Union[DnaSequence, AaSequence, Mixture, DnaOligo, RnaOligo, CustomEntity, UnknownType],
                    InaccessibleResource,
                    UnknownType,
                ]
                if data is None:
                    return data
                if isinstance(data, Unset):
                    return data
                try:

                    def _parse_entity_or_inaccessible_resource(
                        data: Union[Dict[str, Any]]
                    ) -> Union[
                        DnaSequence, AaSequence, Mixture, DnaOligo, RnaOligo, CustomEntity, UnknownType
                    ]:
                        entity_or_inaccessible_resource: Union[
                            DnaSequence, AaSequence, Mixture, DnaOligo, RnaOligo, CustomEntity, UnknownType
                        ]
                        try:
                            if not isinstance(data, dict):
                                raise TypeError()
                            entity = DnaSequence.from_dict(data, strict=True)

                            return entity
                        except:  # noqa: E722
                            pass
                        try:
                            if not isinstance(data, dict):
                                raise TypeError()
                            entity = AaSequence.from_dict(data, strict=True)

                            return entity
                        except:  # noqa: E722
                            pass
                        try:
                            if not isinstance(data, dict):
                                raise TypeError()
                            entity = Mixture.from_dict(data, strict=True)

                            return entity
                        except:  # noqa: E722
                            pass
                        try:
                            if not isinstance(data, dict):
                                raise TypeError()
                            entity = DnaOligo.from_dict(data, strict=True)

                            return entity
                        except:  # noqa: E722
                            pass
                        try:
                            if not isinstance(data, dict):
                                raise TypeError()
                            entity = RnaOligo.from_dict(data, strict=True)

                            return entity
                        except:  # noqa: E722
                            pass
                        try:
                            if not isinstance(data, dict):
                                raise TypeError()
                            entity = CustomEntity.from_dict(data, strict=True)

                            return entity
                        except:  # noqa: E722
                            pass
                        raise ValueError("Unrecognized data type")

                    entity_or_inaccessible_resource = _parse_entity_or_inaccessible_resource(data)

                    return entity_or_inaccessible_resource
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    entity_or_inaccessible_resource = InaccessibleResource.from_dict(data, strict=True)

                    return entity_or_inaccessible_resource
                except:  # noqa: E722
                    pass
                return UnknownType(data)

            entity = _parse_entity(d.pop("entity"))

            return entity

        try:
            entity = get_entity()
        except KeyError:
            if strict:
                raise
            entity = cast(
                Union[
                    Unset,
                    None,
                    Union[DnaSequence, AaSequence, Mixture, DnaOligo, RnaOligo, CustomEntity, UnknownType],
                    InaccessibleResource,
                    UnknownType,
                ],
                UNSET,
            )

        container_content = cls(
            batch=batch,
            concentration=concentration,
            entity=entity,
        )

        container_content.additional_properties = d
        return container_content

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def batch(self) -> Optional[Union[Batch, InaccessibleResource, UnknownType]]:
        if isinstance(self._batch, Unset):
            raise NotPresentError(self, "batch")
        return self._batch

    @batch.setter
    def batch(self, value: Optional[Union[Batch, InaccessibleResource, UnknownType]]) -> None:
        self._batch = value

    @batch.deleter
    def batch(self) -> None:
        self._batch = UNSET

    @property
    def concentration(self) -> Measurement:
        if isinstance(self._concentration, Unset):
            raise NotPresentError(self, "concentration")
        return self._concentration

    @concentration.setter
    def concentration(self, value: Measurement) -> None:
        self._concentration = value

    @concentration.deleter
    def concentration(self) -> None:
        self._concentration = UNSET

    @property
    def entity(
        self,
    ) -> Optional[
        Union[
            Union[DnaSequence, AaSequence, Mixture, DnaOligo, RnaOligo, CustomEntity, UnknownType],
            InaccessibleResource,
            UnknownType,
        ]
    ]:
        if isinstance(self._entity, Unset):
            raise NotPresentError(self, "entity")
        return self._entity

    @entity.setter
    def entity(
        self,
        value: Optional[
            Union[
                Union[DnaSequence, AaSequence, Mixture, DnaOligo, RnaOligo, CustomEntity, UnknownType],
                InaccessibleResource,
                UnknownType,
            ]
        ],
    ) -> None:
        self._entity = value

    @entity.deleter
    def entity(self) -> None:
        self._entity = UNSET



from typing import Any, cast, Dict, List, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.aa_sequence_bulk_upsert_request import AaSequenceBulkUpsertRequest
from ..models.custom_entity_bulk_upsert_request import CustomEntityBulkUpsertRequest
from ..models.dna_sequence_bulk_upsert_request import DnaSequenceBulkUpsertRequest
from ..models.molecule_bulk_upsert_request import MoleculeBulkUpsertRequest
from ..models.oligo_bulk_upsert_request import OligoBulkUpsertRequest
from ..types import UNSET, Unset

T = TypeVar("T", bound="EntitiesBulkUpsertRequest")


@attr.s(auto_attribs=True, repr=False)
class EntitiesBulkUpsertRequest:
    """  """

    _aa_sequences: Union[Unset, List[AaSequenceBulkUpsertRequest]] = UNSET
    _custom_entities: Union[Unset, List[CustomEntityBulkUpsertRequest]] = UNSET
    _dna_oligos: Union[Unset, List[OligoBulkUpsertRequest]] = UNSET
    _dna_sequences: Union[Unset, List[DnaSequenceBulkUpsertRequest]] = UNSET
    _molecules: Union[Unset, List[MoleculeBulkUpsertRequest]] = UNSET
    _rna_oligos: Union[Unset, List[OligoBulkUpsertRequest]] = UNSET

    def __repr__(self):
        fields = []
        fields.append("aa_sequences={}".format(repr(self._aa_sequences)))
        fields.append("custom_entities={}".format(repr(self._custom_entities)))
        fields.append("dna_oligos={}".format(repr(self._dna_oligos)))
        fields.append("dna_sequences={}".format(repr(self._dna_sequences)))
        fields.append("molecules={}".format(repr(self._molecules)))
        fields.append("rna_oligos={}".format(repr(self._rna_oligos)))
        return "EntitiesBulkUpsertRequest({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        aa_sequences: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._aa_sequences, Unset):
            aa_sequences = []
            for aa_sequences_item_data in self._aa_sequences:
                aa_sequences_item = aa_sequences_item_data.to_dict()

                aa_sequences.append(aa_sequences_item)

        custom_entities: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._custom_entities, Unset):
            custom_entities = []
            for custom_entities_item_data in self._custom_entities:
                custom_entities_item = custom_entities_item_data.to_dict()

                custom_entities.append(custom_entities_item)

        dna_oligos: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._dna_oligos, Unset):
            dna_oligos = []
            for dna_oligos_item_data in self._dna_oligos:
                dna_oligos_item = dna_oligos_item_data.to_dict()

                dna_oligos.append(dna_oligos_item)

        dna_sequences: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._dna_sequences, Unset):
            dna_sequences = []
            for dna_sequences_item_data in self._dna_sequences:
                dna_sequences_item = dna_sequences_item_data.to_dict()

                dna_sequences.append(dna_sequences_item)

        molecules: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._molecules, Unset):
            molecules = []
            for molecules_item_data in self._molecules:
                molecules_item = molecules_item_data.to_dict()

                molecules.append(molecules_item)

        rna_oligos: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._rna_oligos, Unset):
            rna_oligos = []
            for rna_oligos_item_data in self._rna_oligos:
                rna_oligos_item = rna_oligos_item_data.to_dict()

                rna_oligos.append(rna_oligos_item)

        field_dict: Dict[str, Any] = {}
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if aa_sequences is not UNSET:
            field_dict["aaSequences"] = aa_sequences
        if custom_entities is not UNSET:
            field_dict["customEntities"] = custom_entities
        if dna_oligos is not UNSET:
            field_dict["dnaOligos"] = dna_oligos
        if dna_sequences is not UNSET:
            field_dict["dnaSequences"] = dna_sequences
        if molecules is not UNSET:
            field_dict["molecules"] = molecules
        if rna_oligos is not UNSET:
            field_dict["rnaOligos"] = rna_oligos

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_aa_sequences() -> Union[Unset, List[AaSequenceBulkUpsertRequest]]:
            aa_sequences = []
            _aa_sequences = d.pop("aaSequences")
            for aa_sequences_item_data in _aa_sequences or []:
                aa_sequences_item = AaSequenceBulkUpsertRequest.from_dict(
                    aa_sequences_item_data, strict=False
                )

                aa_sequences.append(aa_sequences_item)

            return aa_sequences

        try:
            aa_sequences = get_aa_sequences()
        except KeyError:
            if strict:
                raise
            aa_sequences = cast(Union[Unset, List[AaSequenceBulkUpsertRequest]], UNSET)

        def get_custom_entities() -> Union[Unset, List[CustomEntityBulkUpsertRequest]]:
            custom_entities = []
            _custom_entities = d.pop("customEntities")
            for custom_entities_item_data in _custom_entities or []:
                custom_entities_item = CustomEntityBulkUpsertRequest.from_dict(
                    custom_entities_item_data, strict=False
                )

                custom_entities.append(custom_entities_item)

            return custom_entities

        try:
            custom_entities = get_custom_entities()
        except KeyError:
            if strict:
                raise
            custom_entities = cast(Union[Unset, List[CustomEntityBulkUpsertRequest]], UNSET)

        def get_dna_oligos() -> Union[Unset, List[OligoBulkUpsertRequest]]:
            dna_oligos = []
            _dna_oligos = d.pop("dnaOligos")
            for dna_oligos_item_data in _dna_oligos or []:
                dna_oligos_item = OligoBulkUpsertRequest.from_dict(dna_oligos_item_data, strict=False)

                dna_oligos.append(dna_oligos_item)

            return dna_oligos

        try:
            dna_oligos = get_dna_oligos()
        except KeyError:
            if strict:
                raise
            dna_oligos = cast(Union[Unset, List[OligoBulkUpsertRequest]], UNSET)

        def get_dna_sequences() -> Union[Unset, List[DnaSequenceBulkUpsertRequest]]:
            dna_sequences = []
            _dna_sequences = d.pop("dnaSequences")
            for dna_sequences_item_data in _dna_sequences or []:
                dna_sequences_item = DnaSequenceBulkUpsertRequest.from_dict(
                    dna_sequences_item_data, strict=False
                )

                dna_sequences.append(dna_sequences_item)

            return dna_sequences

        try:
            dna_sequences = get_dna_sequences()
        except KeyError:
            if strict:
                raise
            dna_sequences = cast(Union[Unset, List[DnaSequenceBulkUpsertRequest]], UNSET)

        def get_molecules() -> Union[Unset, List[MoleculeBulkUpsertRequest]]:
            molecules = []
            _molecules = d.pop("molecules")
            for molecules_item_data in _molecules or []:
                molecules_item = MoleculeBulkUpsertRequest.from_dict(molecules_item_data, strict=False)

                molecules.append(molecules_item)

            return molecules

        try:
            molecules = get_molecules()
        except KeyError:
            if strict:
                raise
            molecules = cast(Union[Unset, List[MoleculeBulkUpsertRequest]], UNSET)

        def get_rna_oligos() -> Union[Unset, List[OligoBulkUpsertRequest]]:
            rna_oligos = []
            _rna_oligos = d.pop("rnaOligos")
            for rna_oligos_item_data in _rna_oligos or []:
                rna_oligos_item = OligoBulkUpsertRequest.from_dict(rna_oligos_item_data, strict=False)

                rna_oligos.append(rna_oligos_item)

            return rna_oligos

        try:
            rna_oligos = get_rna_oligos()
        except KeyError:
            if strict:
                raise
            rna_oligos = cast(Union[Unset, List[OligoBulkUpsertRequest]], UNSET)

        entities_bulk_upsert_request = cls(
            aa_sequences=aa_sequences,
            custom_entities=custom_entities,
            dna_oligos=dna_oligos,
            dna_sequences=dna_sequences,
            molecules=molecules,
            rna_oligos=rna_oligos,
        )

        return entities_bulk_upsert_request

    @property
    def aa_sequences(self) -> List[AaSequenceBulkUpsertRequest]:
        if isinstance(self._aa_sequences, Unset):
            raise NotPresentError(self, "aa_sequences")
        return self._aa_sequences

    @aa_sequences.setter
    def aa_sequences(self, value: List[AaSequenceBulkUpsertRequest]) -> None:
        self._aa_sequences = value

    @aa_sequences.deleter
    def aa_sequences(self) -> None:
        self._aa_sequences = UNSET

    @property
    def custom_entities(self) -> List[CustomEntityBulkUpsertRequest]:
        if isinstance(self._custom_entities, Unset):
            raise NotPresentError(self, "custom_entities")
        return self._custom_entities

    @custom_entities.setter
    def custom_entities(self, value: List[CustomEntityBulkUpsertRequest]) -> None:
        self._custom_entities = value

    @custom_entities.deleter
    def custom_entities(self) -> None:
        self._custom_entities = UNSET

    @property
    def dna_oligos(self) -> List[OligoBulkUpsertRequest]:
        if isinstance(self._dna_oligos, Unset):
            raise NotPresentError(self, "dna_oligos")
        return self._dna_oligos

    @dna_oligos.setter
    def dna_oligos(self, value: List[OligoBulkUpsertRequest]) -> None:
        self._dna_oligos = value

    @dna_oligos.deleter
    def dna_oligos(self) -> None:
        self._dna_oligos = UNSET

    @property
    def dna_sequences(self) -> List[DnaSequenceBulkUpsertRequest]:
        if isinstance(self._dna_sequences, Unset):
            raise NotPresentError(self, "dna_sequences")
        return self._dna_sequences

    @dna_sequences.setter
    def dna_sequences(self, value: List[DnaSequenceBulkUpsertRequest]) -> None:
        self._dna_sequences = value

    @dna_sequences.deleter
    def dna_sequences(self) -> None:
        self._dna_sequences = UNSET

    @property
    def molecules(self) -> List[MoleculeBulkUpsertRequest]:
        if isinstance(self._molecules, Unset):
            raise NotPresentError(self, "molecules")
        return self._molecules

    @molecules.setter
    def molecules(self, value: List[MoleculeBulkUpsertRequest]) -> None:
        self._molecules = value

    @molecules.deleter
    def molecules(self) -> None:
        self._molecules = UNSET

    @property
    def rna_oligos(self) -> List[OligoBulkUpsertRequest]:
        if isinstance(self._rna_oligos, Unset):
            raise NotPresentError(self, "rna_oligos")
        return self._rna_oligos

    @rna_oligos.setter
    def rna_oligos(self, value: List[OligoBulkUpsertRequest]) -> None:
        self._rna_oligos = value

    @rna_oligos.deleter
    def rna_oligos(self) -> None:
        self._rna_oligos = UNSET



from enum import Enum
from functools import lru_cache
from typing import cast

from ..extensions import Enums


class DeprecatedContainerVolumeForInputUnits(Enums.KnownString):
    PL = "pL"
    NL = "nL"
    UL = "uL"
    ML = "mL"
    L = "L"

    def __str__(self) -> str:
        return str(self.value)

    @staticmethod
    @lru_cache(maxsize=None)
    def of_unknown(val: str) -> "DeprecatedContainerVolumeForInputUnits":
        if not isinstance(val, str):
            raise ValueError(
                f"Value of DeprecatedContainerVolumeForInputUnits must be a string (encountered: {val})"
            )
        newcls = Enum("DeprecatedContainerVolumeForInputUnits", {"_UNKNOWN": val}, type=Enums.UnknownString)  # type: ignore
        return cast(DeprecatedContainerVolumeForInputUnits, getattr(newcls, "_UNKNOWN"))



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.entry_link_type import EntryLinkType
from ..types import UNSET, Unset

T = TypeVar("T", bound="EntryLink")


@attr.s(auto_attribs=True, repr=False)
class EntryLink:
    """Links are contained within notes to reference resources that live outside of the entry. A link can target an external resource via an http(s):// hyperlink or a Benchling resource via @-mentions and drag-n-drop."""

    _id: Union[Unset, str] = UNSET
    _type: Union[Unset, EntryLinkType] = UNSET
    _web_url: Union[Unset, None, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("id={}".format(repr(self._id)))
        fields.append("type={}".format(repr(self._type)))
        fields.append("web_url={}".format(repr(self._web_url)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "EntryLink({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        id = self._id
        type: Union[Unset, int] = UNSET
        if not isinstance(self._type, Unset):
            type = self._type.value

        web_url = self._web_url

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if id is not UNSET:
            field_dict["id"] = id
        if type is not UNSET:
            field_dict["type"] = type
        if web_url is not UNSET:
            field_dict["webURL"] = web_url

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_id() -> Union[Unset, str]:
            id = d.pop("id")
            return id

        try:
            id = get_id()
        except KeyError:
            if strict:
                raise
            id = cast(Union[Unset, str], UNSET)

        def get_type() -> Union[Unset, EntryLinkType]:
            type = UNSET
            _type = d.pop("type")
            if _type is not None and _type is not UNSET:
                try:
                    type = EntryLinkType(_type)
                except ValueError:
                    type = EntryLinkType.of_unknown(_type)

            return type

        try:
            type = get_type()
        except KeyError:
            if strict:
                raise
            type = cast(Union[Unset, EntryLinkType], UNSET)

        def get_web_url() -> Union[Unset, None, str]:
            web_url = d.pop("webURL")
            return web_url

        try:
            web_url = get_web_url()
        except KeyError:
            if strict:
                raise
            web_url = cast(Union[Unset, None, str], UNSET)

        entry_link = cls(
            id=id,
            type=type,
            web_url=web_url,
        )

        entry_link.additional_properties = d
        return entry_link

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def id(self) -> str:
        """For linked Benchling resources, this will be the ID of that resource (e.g., 'seq_RhYGVnHF'). Omitted for "link" types."""
        if isinstance(self._id, Unset):
            raise NotPresentError(self, "id")
        return self._id

    @id.setter
    def id(self, value: str) -> None:
        self._id = value

    @id.deleter
    def id(self) -> None:
        self._id = UNSET

    @property
    def type(self) -> EntryLinkType:
        """The type of resource being linked. For hyperlinks: 'link'. For linked Benchling resources, one of: 'user', 'request', 'entry', 'stage_entry', 'protocol', 'workflow', 'custom_entity', 'aa_sequence', 'dna_sequence', 'batch', 'box', 'container', 'location', 'plate'."""
        if isinstance(self._type, Unset):
            raise NotPresentError(self, "type")
        return self._type

    @type.setter
    def type(self, value: EntryLinkType) -> None:
        self._type = value

    @type.deleter
    def type(self) -> None:
        self._type = UNSET

    @property
    def web_url(self) -> Optional[str]:
        """Canonical URL of the linked Benchling resource (if you have at least READ authorization for that resource), or the explicit URL provided as hyperlink for "link" types. Note: locations do not currently have a URL."""
        if isinstance(self._web_url, Unset):
            raise NotPresentError(self, "web_url")
        return self._web_url

    @web_url.setter
    def web_url(self, value: Optional[str]) -> None:
        self._web_url = value

    @web_url.deleter
    def web_url(self) -> None:
        self._web_url = UNSET



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.barcode_validation_result import BarcodeValidationResult
from ..types import UNSET, Unset

T = TypeVar("T", bound="BarcodeValidationResults")


@attr.s(auto_attribs=True, repr=False)
class BarcodeValidationResults:
    """  """

    _validation_results: Union[Unset, List[BarcodeValidationResult]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("validation_results={}".format(repr(self._validation_results)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "BarcodeValidationResults({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        validation_results: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._validation_results, Unset):
            validation_results = []
            for validation_results_item_data in self._validation_results:
                validation_results_item = validation_results_item_data.to_dict()

                validation_results.append(validation_results_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if validation_results is not UNSET:
            field_dict["validationResults"] = validation_results

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_validation_results() -> Union[Unset, List[BarcodeValidationResult]]:
            validation_results = []
            _validation_results = d.pop("validationResults")
            for validation_results_item_data in _validation_results or []:
                validation_results_item = BarcodeValidationResult.from_dict(
                    validation_results_item_data, strict=False
                )

                validation_results.append(validation_results_item)

            return validation_results

        try:
            validation_results = get_validation_results()
        except KeyError:
            if strict:
                raise
            validation_results = cast(Union[Unset, List[BarcodeValidationResult]], UNSET)

        barcode_validation_results = cls(
            validation_results=validation_results,
        )

        barcode_validation_results.additional_properties = d
        return barcode_validation_results

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def validation_results(self) -> List[BarcodeValidationResult]:
        if isinstance(self._validation_results, Unset):
            raise NotPresentError(self, "validation_results")
        return self._validation_results

    @validation_results.setter
    def validation_results(self, value: List[BarcodeValidationResult]) -> None:
        self._validation_results = value

    @validation_results.deleter
    def validation_results(self) -> None:
        self._validation_results = UNSET



from typing import Any, cast, Dict, Type, TypeVar

import attr

from ..extensions import NotPresentError
from ..types import UNSET, Unset

T = TypeVar("T", bound="FolderCreate")


@attr.s(auto_attribs=True, repr=False)
class FolderCreate:
    """  """

    _name: str
    _parent_folder_id: str

    def __repr__(self):
        fields = []
        fields.append("name={}".format(repr(self._name)))
        fields.append("parent_folder_id={}".format(repr(self._parent_folder_id)))
        return "FolderCreate({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        name = self._name
        parent_folder_id = self._parent_folder_id

        field_dict: Dict[str, Any] = {}
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if name is not UNSET:
            field_dict["name"] = name
        if parent_folder_id is not UNSET:
            field_dict["parentFolderId"] = parent_folder_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_name() -> str:
            name = d.pop("name")
            return name

        try:
            name = get_name()
        except KeyError:
            if strict:
                raise
            name = cast(str, UNSET)

        def get_parent_folder_id() -> str:
            parent_folder_id = d.pop("parentFolderId")
            return parent_folder_id

        try:
            parent_folder_id = get_parent_folder_id()
        except KeyError:
            if strict:
                raise
            parent_folder_id = cast(str, UNSET)

        folder_create = cls(
            name=name,
            parent_folder_id=parent_folder_id,
        )

        return folder_create

    @property
    def name(self) -> str:
        """ The name of the new folder. """
        if isinstance(self._name, Unset):
            raise NotPresentError(self, "name")
        return self._name

    @name.setter
    def name(self, value: str) -> None:
        self._name = value

    @property
    def parent_folder_id(self) -> str:
        """ The ID of the parent folder. """
        if isinstance(self._parent_folder_id, Unset):
            raise NotPresentError(self, "parent_folder_id")
        return self._parent_folder_id

    @parent_folder_id.setter
    def parent_folder_id(self, value: str) -> None:
        self._parent_folder_id = value



from typing import Any, cast, Dict, List, Type, TypeVar

import attr

from ..extensions import NotPresentError
from ..types import UNSET, Unset

T = TypeVar("T", bound="ProjectsUnarchive")


@attr.s(auto_attribs=True, repr=False)
class ProjectsUnarchive:
    """  """

    _project_ids: List[str]

    def __repr__(self):
        fields = []
        fields.append("project_ids={}".format(repr(self._project_ids)))
        return "ProjectsUnarchive({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        project_ids = self._project_ids

        field_dict: Dict[str, Any] = {}
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if project_ids is not UNSET:
            field_dict["projectIds"] = project_ids

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_project_ids() -> List[str]:
            project_ids = cast(List[str], d.pop("projectIds"))

            return project_ids

        try:
            project_ids = get_project_ids()
        except KeyError:
            if strict:
                raise
            project_ids = cast(List[str], UNSET)

        projects_unarchive = cls(
            project_ids=project_ids,
        )

        return projects_unarchive

    @property
    def project_ids(self) -> List[str]:
        """ A list of project IDs to unarchive. """
        if isinstance(self._project_ids, Unset):
            raise NotPresentError(self, "project_ids")
        return self._project_ids

    @project_ids.setter
    def project_ids(self, value: List[str]) -> None:
        self._project_ids = value



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..types import UNSET, Unset

T = TypeVar("T", bound="BaseDependencyLink")


@attr.s(auto_attribs=True, repr=False)
class BaseDependencyLink:
    """  """

    _name: str
    _resource_id: Optional[str]
    _description: Union[Unset, None, str] = UNSET
    _required_config: Union[Unset, bool] = False
    _resource_name: Union[Unset, None, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("name={}".format(repr(self._name)))
        fields.append("resource_id={}".format(repr(self._resource_id)))
        fields.append("description={}".format(repr(self._description)))
        fields.append("required_config={}".format(repr(self._required_config)))
        fields.append("resource_name={}".format(repr(self._resource_name)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "BaseDependencyLink({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        name = self._name
        resource_id = self._resource_id
        description = self._description
        required_config = self._required_config
        resource_name = self._resource_name

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if name is not UNSET:
            field_dict["name"] = name
        if resource_id is not UNSET:
            field_dict["resourceId"] = resource_id
        if description is not UNSET:
            field_dict["description"] = description
        if required_config is not UNSET:
            field_dict["requiredConfig"] = required_config
        if resource_name is not UNSET:
            field_dict["resourceName"] = resource_name

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_name() -> str:
            name = d.pop("name")
            return name

        try:
            name = get_name()
        except KeyError:
            if strict:
                raise
            name = cast(str, UNSET)

        def get_resource_id() -> Optional[str]:
            resource_id = d.pop("resourceId")
            return resource_id

        try:
            resource_id = get_resource_id()
        except KeyError:
            if strict:
                raise
            resource_id = cast(Optional[str], UNSET)

        def get_description() -> Union[Unset, None, str]:
            description = d.pop("description")
            return description

        try:
            description = get_description()
        except KeyError:
            if strict:
                raise
            description = cast(Union[Unset, None, str], UNSET)

        def get_required_config() -> Union[Unset, bool]:
            required_config = d.pop("requiredConfig")
            return required_config

        try:
            required_config = get_required_config()
        except KeyError:
            if strict:
                raise
            required_config = cast(Union[Unset, bool], UNSET)

        def get_resource_name() -> Union[Unset, None, str]:
            resource_name = d.pop("resourceName")
            return resource_name

        try:
            resource_name = get_resource_name()
        except KeyError:
            if strict:
                raise
            resource_name = cast(Union[Unset, None, str], UNSET)

        base_dependency_link = cls(
            name=name,
            resource_id=resource_id,
            description=description,
            required_config=required_config,
            resource_name=resource_name,
        )

        base_dependency_link.additional_properties = d
        return base_dependency_link

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def name(self) -> str:
        if isinstance(self._name, Unset):
            raise NotPresentError(self, "name")
        return self._name

    @name.setter
    def name(self, value: str) -> None:
        self._name = value

    @property
    def resource_id(self) -> Optional[str]:
        if isinstance(self._resource_id, Unset):
            raise NotPresentError(self, "resource_id")
        return self._resource_id

    @resource_id.setter
    def resource_id(self, value: Optional[str]) -> None:
        self._resource_id = value

    @property
    def description(self) -> Optional[str]:
        if isinstance(self._description, Unset):
            raise NotPresentError(self, "description")
        return self._description

    @description.setter
    def description(self, value: Optional[str]) -> None:
        self._description = value

    @description.deleter
    def description(self) -> None:
        self._description = UNSET

    @property
    def required_config(self) -> bool:
        if isinstance(self._required_config, Unset):
            raise NotPresentError(self, "required_config")
        return self._required_config

    @required_config.setter
    def required_config(self, value: bool) -> None:
        self._required_config = value

    @required_config.deleter
    def required_config(self) -> None:
        self._required_config = UNSET

    @property
    def resource_name(self) -> Optional[str]:
        if isinstance(self._resource_name, Unset):
            raise NotPresentError(self, "resource_name")
        return self._resource_name

    @resource_name.setter
    def resource_name(self, value: Optional[str]) -> None:
        self._resource_name = value

    @resource_name.deleter
    def resource_name(self) -> None:
        self._resource_name = UNSET



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..types import UNSET, Unset

T = TypeVar("T", bound="CustomEntitiesArchivalChange")


@attr.s(auto_attribs=True, repr=False)
class CustomEntitiesArchivalChange:
    """IDs of all items that were archived or unarchived, grouped by resource type. This includes the IDs of custom entities along with any IDs of batches that were archived (or unarchived)."""

    _batch_ids: Union[Unset, List[str]] = UNSET
    _custom_entity_ids: Union[Unset, List[str]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("batch_ids={}".format(repr(self._batch_ids)))
        fields.append("custom_entity_ids={}".format(repr(self._custom_entity_ids)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "CustomEntitiesArchivalChange({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        batch_ids: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._batch_ids, Unset):
            batch_ids = self._batch_ids

        custom_entity_ids: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._custom_entity_ids, Unset):
            custom_entity_ids = self._custom_entity_ids

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if batch_ids is not UNSET:
            field_dict["batchIds"] = batch_ids
        if custom_entity_ids is not UNSET:
            field_dict["customEntityIds"] = custom_entity_ids

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_batch_ids() -> Union[Unset, List[str]]:
            batch_ids = cast(List[str], d.pop("batchIds"))

            return batch_ids

        try:
            batch_ids = get_batch_ids()
        except KeyError:
            if strict:
                raise
            batch_ids = cast(Union[Unset, List[str]], UNSET)

        def get_custom_entity_ids() -> Union[Unset, List[str]]:
            custom_entity_ids = cast(List[str], d.pop("customEntityIds"))

            return custom_entity_ids

        try:
            custom_entity_ids = get_custom_entity_ids()
        except KeyError:
            if strict:
                raise
            custom_entity_ids = cast(Union[Unset, List[str]], UNSET)

        custom_entities_archival_change = cls(
            batch_ids=batch_ids,
            custom_entity_ids=custom_entity_ids,
        )

        custom_entities_archival_change.additional_properties = d
        return custom_entities_archival_change

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def batch_ids(self) -> List[str]:
        if isinstance(self._batch_ids, Unset):
            raise NotPresentError(self, "batch_ids")
        return self._batch_ids

    @batch_ids.setter
    def batch_ids(self, value: List[str]) -> None:
        self._batch_ids = value

    @batch_ids.deleter
    def batch_ids(self) -> None:
        self._batch_ids = UNSET

    @property
    def custom_entity_ids(self) -> List[str]:
        if isinstance(self._custom_entity_ids, Unset):
            raise NotPresentError(self, "custom_entity_ids")
        return self._custom_entity_ids

    @custom_entity_ids.setter
    def custom_entity_ids(self, value: List[str]) -> None:
        self._custom_entity_ids = value

    @custom_entity_ids.deleter
    def custom_entity_ids(self) -> None:
        self._custom_entity_ids = UNSET



from typing import Any, cast, Dict, List, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.oligo_create import OligoCreate
from ..types import UNSET, Unset

T = TypeVar("T", bound="OligosBulkCreateRequest")


@attr.s(auto_attribs=True, repr=False)
class OligosBulkCreateRequest:
    """  """

    _oligos: Union[Unset, List[OligoCreate]] = UNSET

    def __repr__(self):
        fields = []
        fields.append("oligos={}".format(repr(self._oligos)))
        return "OligosBulkCreateRequest({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        oligos: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._oligos, Unset):
            oligos = []
            for oligos_item_data in self._oligos:
                oligos_item = oligos_item_data.to_dict()

                oligos.append(oligos_item)

        field_dict: Dict[str, Any] = {}
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if oligos is not UNSET:
            field_dict["oligos"] = oligos

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_oligos() -> Union[Unset, List[OligoCreate]]:
            oligos = []
            _oligos = d.pop("oligos")
            for oligos_item_data in _oligos or []:
                oligos_item = OligoCreate.from_dict(oligos_item_data, strict=False)

                oligos.append(oligos_item)

            return oligos

        try:
            oligos = get_oligos()
        except KeyError:
            if strict:
                raise
            oligos = cast(Union[Unset, List[OligoCreate]], UNSET)

        oligos_bulk_create_request = cls(
            oligos=oligos,
        )

        return oligos_bulk_create_request

    @property
    def oligos(self) -> List[OligoCreate]:
        if isinstance(self._oligos, Unset):
            raise NotPresentError(self, "oligos")
        return self._oligos

    @oligos.setter
    def oligos(self, value: List[OligoCreate]) -> None:
        self._oligos = value

    @oligos.deleter
    def oligos(self) -> None:
        self._oligos = UNSET



import datetime
from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..extensions import NotPresentError
from ..models.archive_record import ArchiveRecord
from ..models.automation_file_automation_file_config import AutomationFileAutomationFileConfig
from ..models.automation_file_status import AutomationFileStatus
from ..models.automation_progress_stats import AutomationProgressStats
from ..models.blob import Blob
from ..models.lab_automation_transform import LabAutomationTransform
from ..types import UNSET, Unset

T = TypeVar("T", bound="AutomationOutputProcessor")


@attr.s(auto_attribs=True, repr=False)
class AutomationOutputProcessor:
    """  """

    _api_url: Union[Unset, str] = UNSET
    _archive_record: Union[Unset, ArchiveRecord] = UNSET
    _complete_with_errors: Union[Unset, bool] = UNSET
    _created_at: Union[Unset, datetime.datetime] = UNSET
    _error_file: Union[Unset, None, Blob] = UNSET
    _id: Union[Unset, str] = UNSET
    _modified_at: Union[Unset, datetime.datetime] = UNSET
    _progress_stats: Union[Unset, AutomationProgressStats] = UNSET
    _transforms: Union[Unset, List[Optional[LabAutomationTransform]]] = UNSET
    _assay_run_id: Union[Unset, str] = UNSET
    _automation_file_config: Union[Unset, AutomationFileAutomationFileConfig] = UNSET
    _file: Union[Unset, None, Blob] = UNSET
    _status: Union[Unset, AutomationFileStatus] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("api_url={}".format(repr(self._api_url)))
        fields.append("archive_record={}".format(repr(self._archive_record)))
        fields.append("complete_with_errors={}".format(repr(self._complete_with_errors)))
        fields.append("created_at={}".format(repr(self._created_at)))
        fields.append("error_file={}".format(repr(self._error_file)))
        fields.append("id={}".format(repr(self._id)))
        fields.append("modified_at={}".format(repr(self._modified_at)))
        fields.append("progress_stats={}".format(repr(self._progress_stats)))
        fields.append("transforms={}".format(repr(self._transforms)))
        fields.append("assay_run_id={}".format(repr(self._assay_run_id)))
        fields.append("automation_file_config={}".format(repr(self._automation_file_config)))
        fields.append("file={}".format(repr(self._file)))
        fields.append("status={}".format(repr(self._status)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "AutomationOutputProcessor({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        api_url = self._api_url
        archive_record: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self._archive_record, Unset):
            archive_record = self._archive_record.to_dict()

        complete_with_errors = self._complete_with_errors
        created_at: Union[Unset, str] = UNSET
        if not isinstance(self._created_at, Unset):
            created_at = self._created_at.isoformat()

        error_file: Union[Unset, None, Dict[str, Any]] = UNSET
        if not isinstance(self._error_file, Unset):
            error_file = self._error_file.to_dict() if self._error_file else None

        id = self._id
        modified_at: Union[Unset, str] = UNSET
        if not isinstance(self._modified_at, Unset):
            modified_at = self._modified_at.isoformat()

        progress_stats: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self._progress_stats, Unset):
            progress_stats = self._progress_stats.to_dict()

        transforms: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._transforms, Unset):
            transforms = []
            for transforms_item_data in self._transforms:
                transforms_item = transforms_item_data.to_dict() if transforms_item_data else None

                transforms.append(transforms_item)

        assay_run_id = self._assay_run_id
        automation_file_config: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self._automation_file_config, Unset):
            automation_file_config = self._automation_file_config.to_dict()

        file: Union[Unset, None, Dict[str, Any]] = UNSET
        if not isinstance(self._file, Unset):
            file = self._file.to_dict() if self._file else None

        status: Union[Unset, int] = UNSET
        if not isinstance(self._status, Unset):
            status = self._status.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if api_url is not UNSET:
            field_dict["apiURL"] = api_url
        if archive_record is not UNSET:
            field_dict["archiveRecord"] = archive_record
        if complete_with_errors is not UNSET:
            field_dict["completeWithErrors"] = complete_with_errors
        if created_at is not UNSET:
            field_dict["createdAt"] = created_at
        if error_file is not UNSET:
            field_dict["errorFile"] = error_file
        if id is not UNSET:
            field_dict["id"] = id
        if modified_at is not UNSET:
            field_dict["modifiedAt"] = modified_at
        if progress_stats is not UNSET:
            field_dict["progressStats"] = progress_stats
        if transforms is not UNSET:
            field_dict["transforms"] = transforms
        if assay_run_id is not UNSET:
            field_dict["assayRunId"] = assay_run_id
        if automation_file_config is not UNSET:
            field_dict["automationFileConfig"] = automation_file_config
        if file is not UNSET:
            field_dict["file"] = file
        if status is not UNSET:
            field_dict["status"] = status

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_api_url() -> Union[Unset, str]:
            api_url = d.pop("apiURL")
            return api_url

        try:
            api_url = get_api_url()
        except KeyError:
            if strict:
                raise
            api_url = cast(Union[Unset, str], UNSET)

        def get_archive_record() -> Union[Unset, ArchiveRecord]:
            archive_record: Union[Unset, Union[Unset, ArchiveRecord]] = UNSET
            _archive_record = d.pop("archiveRecord")

            if not isinstance(_archive_record, Unset):
                archive_record = ArchiveRecord.from_dict(_archive_record)

            return archive_record

        try:
            archive_record = get_archive_record()
        except KeyError:
            if strict:
                raise
            archive_record = cast(Union[Unset, ArchiveRecord], UNSET)

        def get_complete_with_errors() -> Union[Unset, bool]:
            complete_with_errors = d.pop("completeWithErrors")
            return complete_with_errors

        try:
            complete_with_errors = get_complete_with_errors()
        except KeyError:
            if strict:
                raise
            complete_with_errors = cast(Union[Unset, bool], UNSET)

        def get_created_at() -> Union[Unset, datetime.datetime]:
            created_at: Union[Unset, datetime.datetime] = UNSET
            _created_at = d.pop("createdAt")
            if _created_at is not None and not isinstance(_created_at, Unset):
                created_at = isoparse(cast(str, _created_at))

            return created_at

        try:
            created_at = get_created_at()
        except KeyError:
            if strict:
                raise
            created_at = cast(Union[Unset, datetime.datetime], UNSET)

        def get_error_file() -> Union[Unset, None, Blob]:
            error_file = None
            _error_file = d.pop("errorFile")

            if _error_file is not None and not isinstance(_error_file, Unset):
                error_file = Blob.from_dict(_error_file)

            return error_file

        try:
            error_file = get_error_file()
        except KeyError:
            if strict:
                raise
            error_file = cast(Union[Unset, None, Blob], UNSET)

        def get_id() -> Union[Unset, str]:
            id = d.pop("id")
            return id

        try:
            id = get_id()
        except KeyError:
            if strict:
                raise
            id = cast(Union[Unset, str], UNSET)

        def get_modified_at() -> Union[Unset, datetime.datetime]:
            modified_at: Union[Unset, datetime.datetime] = UNSET
            _modified_at = d.pop("modifiedAt")
            if _modified_at is not None and not isinstance(_modified_at, Unset):
                modified_at = isoparse(cast(str, _modified_at))

            return modified_at

        try:
            modified_at = get_modified_at()
        except KeyError:
            if strict:
                raise
            modified_at = cast(Union[Unset, datetime.datetime], UNSET)

        def get_progress_stats() -> Union[Unset, AutomationProgressStats]:
            progress_stats: Union[Unset, Union[Unset, AutomationProgressStats]] = UNSET
            _progress_stats = d.pop("progressStats")

            if not isinstance(_progress_stats, Unset):
                progress_stats = AutomationProgressStats.from_dict(_progress_stats)

            return progress_stats

        try:
            progress_stats = get_progress_stats()
        except KeyError:
            if strict:
                raise
            progress_stats = cast(Union[Unset, AutomationProgressStats], UNSET)

        def get_transforms() -> Union[Unset, List[Optional[LabAutomationTransform]]]:
            transforms = []
            _transforms = d.pop("transforms")
            for transforms_item_data in _transforms or []:
                transforms_item = None
                _transforms_item = transforms_item_data

                if _transforms_item is not None:
                    transforms_item = LabAutomationTransform.from_dict(_transforms_item)

                transforms.append(transforms_item)

            return transforms

        try:
            transforms = get_transforms()
        except KeyError:
            if strict:
                raise
            transforms = cast(Union[Unset, List[Optional[LabAutomationTransform]]], UNSET)

        def get_assay_run_id() -> Union[Unset, str]:
            assay_run_id = d.pop("assayRunId")
            return assay_run_id

        try:
            assay_run_id = get_assay_run_id()
        except KeyError:
            if strict:
                raise
            assay_run_id = cast(Union[Unset, str], UNSET)

        def get_automation_file_config() -> Union[Unset, AutomationFileAutomationFileConfig]:
            automation_file_config: Union[Unset, Union[Unset, AutomationFileAutomationFileConfig]] = UNSET
            _automation_file_config = d.pop("automationFileConfig")

            if not isinstance(_automation_file_config, Unset):
                automation_file_config = AutomationFileAutomationFileConfig.from_dict(_automation_file_config)

            return automation_file_config

        try:
            automation_file_config = get_automation_file_config()
        except KeyError:
            if strict:
                raise
            automation_file_config = cast(Union[Unset, AutomationFileAutomationFileConfig], UNSET)

        def get_file() -> Union[Unset, None, Blob]:
            file = None
            _file = d.pop("file")

            if _file is not None and not isinstance(_file, Unset):
                file = Blob.from_dict(_file)

            return file

        try:
            file = get_file()
        except KeyError:
            if strict:
                raise
            file = cast(Union[Unset, None, Blob], UNSET)

        def get_status() -> Union[Unset, AutomationFileStatus]:
            status = UNSET
            _status = d.pop("status")
            if _status is not None and _status is not UNSET:
                try:
                    status = AutomationFileStatus(_status)
                except ValueError:
                    status = AutomationFileStatus.of_unknown(_status)

            return status

        try:
            status = get_status()
        except KeyError:
            if strict:
                raise
            status = cast(Union[Unset, AutomationFileStatus], UNSET)

        automation_output_processor = cls(
            api_url=api_url,
            archive_record=archive_record,
            complete_with_errors=complete_with_errors,
            created_at=created_at,
            error_file=error_file,
            id=id,
            modified_at=modified_at,
            progress_stats=progress_stats,
            transforms=transforms,
            assay_run_id=assay_run_id,
            automation_file_config=automation_file_config,
            file=file,
            status=status,
        )

        automation_output_processor.additional_properties = d
        return automation_output_processor

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def api_url(self) -> str:
        """ The canonical url of the Automation Output Processor in the API. """
        if isinstance(self._api_url, Unset):
            raise NotPresentError(self, "api_url")
        return self._api_url

    @api_url.setter
    def api_url(self, value: str) -> None:
        self._api_url = value

    @api_url.deleter
    def api_url(self) -> None:
        self._api_url = UNSET

    @property
    def archive_record(self) -> ArchiveRecord:
        if isinstance(self._archive_record, Unset):
            raise NotPresentError(self, "archive_record")
        return self._archive_record

    @archive_record.setter
    def archive_record(self, value: ArchiveRecord) -> None:
        self._archive_record = value

    @archive_record.deleter
    def archive_record(self) -> None:
        self._archive_record = UNSET

    @property
    def complete_with_errors(self) -> bool:
        """ Specifies whether file processing should complete with errors. False means any error in output file processing will result in no actions being committed. True means that if row-level errors occur, then failing rows and their errors will be saved to errorFile, and actions from successful rows will be committed. """
        if isinstance(self._complete_with_errors, Unset):
            raise NotPresentError(self, "complete_with_errors")
        return self._complete_with_errors

    @complete_with_errors.setter
    def complete_with_errors(self, value: bool) -> None:
        self._complete_with_errors = value

    @complete_with_errors.deleter
    def complete_with_errors(self) -> None:
        self._complete_with_errors = UNSET

    @property
    def created_at(self) -> datetime.datetime:
        """ DateTime the Automation Output Processor was created """
        if isinstance(self._created_at, Unset):
            raise NotPresentError(self, "created_at")
        return self._created_at

    @created_at.setter
    def created_at(self, value: datetime.datetime) -> None:
        self._created_at = value

    @created_at.deleter
    def created_at(self) -> None:
        self._created_at = UNSET

    @property
    def error_file(self) -> Optional[Blob]:
        if isinstance(self._error_file, Unset):
            raise NotPresentError(self, "error_file")
        return self._error_file

    @error_file.setter
    def error_file(self, value: Optional[Blob]) -> None:
        self._error_file = value

    @error_file.deleter
    def error_file(self) -> None:
        self._error_file = UNSET

    @property
    def id(self) -> str:
        if isinstance(self._id, Unset):
            raise NotPresentError(self, "id")
        return self._id

    @id.setter
    def id(self, value: str) -> None:
        self._id = value

    @id.deleter
    def id(self) -> None:
        self._id = UNSET

    @property
    def modified_at(self) -> datetime.datetime:
        """ DateTime the Automation Output Processor was last modified """
        if isinstance(self._modified_at, Unset):
            raise NotPresentError(self, "modified_at")
        return self._modified_at

    @modified_at.setter
    def modified_at(self, value: datetime.datetime) -> None:
        self._modified_at = value

    @modified_at.deleter
    def modified_at(self) -> None:
        self._modified_at = UNSET

    @property
    def progress_stats(self) -> AutomationProgressStats:
        """ Processing progress information. """
        if isinstance(self._progress_stats, Unset):
            raise NotPresentError(self, "progress_stats")
        return self._progress_stats

    @progress_stats.setter
    def progress_stats(self, value: AutomationProgressStats) -> None:
        self._progress_stats = value

    @progress_stats.deleter
    def progress_stats(self) -> None:
        self._progress_stats = UNSET

    @property
    def transforms(self) -> List[Optional[LabAutomationTransform]]:
        if isinstance(self._transforms, Unset):
            raise NotPresentError(self, "transforms")
        return self._transforms

    @transforms.setter
    def transforms(self, value: List[Optional[LabAutomationTransform]]) -> None:
        self._transforms = value

    @transforms.deleter
    def transforms(self) -> None:
        self._transforms = UNSET

    @property
    def assay_run_id(self) -> str:
        if isinstance(self._assay_run_id, Unset):
            raise NotPresentError(self, "assay_run_id")
        return self._assay_run_id

    @assay_run_id.setter
    def assay_run_id(self, value: str) -> None:
        self._assay_run_id = value

    @assay_run_id.deleter
    def assay_run_id(self) -> None:
        self._assay_run_id = UNSET

    @property
    def automation_file_config(self) -> AutomationFileAutomationFileConfig:
        if isinstance(self._automation_file_config, Unset):
            raise NotPresentError(self, "automation_file_config")
        return self._automation_file_config

    @automation_file_config.setter
    def automation_file_config(self, value: AutomationFileAutomationFileConfig) -> None:
        self._automation_file_config = value

    @automation_file_config.deleter
    def automation_file_config(self) -> None:
        self._automation_file_config = UNSET

    @property
    def file(self) -> Optional[Blob]:
        if isinstance(self._file, Unset):
            raise NotPresentError(self, "file")
        return self._file

    @file.setter
    def file(self, value: Optional[Blob]) -> None:
        self._file = value

    @file.deleter
    def file(self) -> None:
        self._file = UNSET

    @property
    def status(self) -> AutomationFileStatus:
        if isinstance(self._status, Unset):
            raise NotPresentError(self, "status")
        return self._status

    @status.setter
    def status(self, value: AutomationFileStatus) -> None:
        self._status = value

    @status.deleter
    def status(self) -> None:
        self._status = UNSET



from enum import Enum
from functools import lru_cache
from typing import cast

from ..extensions import Enums


class SchemaDependencyTypes(Enums.KnownString):
    CONTAINER_SCHEMA = "container_schema"
    PLATE_SCHEMA = "plate_schema"
    LOCATION_SCHEMA = "location_schema"
    BOX_SCHEMA = "box_schema"
    RUN_SCHEMA = "run_schema"
    RESULT_SCHEMA = "result_schema"
    REQUEST_SCHEMA = "request_schema"
    ENTRY_SCHEMA = "entry_schema"
    WORKFLOW_TASK_SCHEMA = "workflow_task_schema"

    def __str__(self) -> str:
        return str(self.value)

    @staticmethod
    @lru_cache(maxsize=None)
    def of_unknown(val: str) -> "SchemaDependencyTypes":
        if not isinstance(val, str):
            raise ValueError(f"Value of SchemaDependencyTypes must be a string (encountered: {val})")
        newcls = Enum("SchemaDependencyTypes", {"_UNKNOWN": val}, type=Enums.UnknownString)  # type: ignore
        return cast(SchemaDependencyTypes, getattr(newcls, "_UNKNOWN"))



from typing import Any, cast, Dict, List, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError, UnknownType
from ..models.dropdown_dependency_link import DropdownDependencyLink
from ..models.entity_schema_dependency_link import EntitySchemaDependencyLink
from ..models.resource_dependency_link import ResourceDependencyLink
from ..models.scalar_config import ScalarConfig
from ..models.schema_dependency_link import SchemaDependencyLink
from ..models.secure_text_config import SecureTextConfig
from ..models.workflow_task_schema_dependency_link import WorkflowTaskSchemaDependencyLink
from ..types import UNSET, Unset

T = TypeVar("T", bound="BenchlingAppCreate")


@attr.s(auto_attribs=True, repr=False)
class BenchlingAppCreate:
    """  """

    _description: Union[Unset, str] = UNSET
    _name: Union[Unset, str] = UNSET
    _configuration: Union[
        Unset,
        List[
            Union[
                SchemaDependencyLink,
                EntitySchemaDependencyLink,
                WorkflowTaskSchemaDependencyLink,
                DropdownDependencyLink,
                ResourceDependencyLink,
                ScalarConfig,
                SecureTextConfig,
                UnknownType,
            ]
        ],
    ] = UNSET

    def __repr__(self):
        fields = []
        fields.append("description={}".format(repr(self._description)))
        fields.append("name={}".format(repr(self._name)))
        fields.append("configuration={}".format(repr(self._configuration)))
        return "BenchlingAppCreate({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        description = self._description
        name = self._name
        configuration: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._configuration, Unset):
            configuration = []
            for configuration_item_data in self._configuration:
                if isinstance(configuration_item_data, UnknownType):
                    configuration_item = configuration_item_data.value
                elif isinstance(configuration_item_data, SchemaDependencyLink):
                    configuration_item = configuration_item_data.to_dict()

                elif isinstance(configuration_item_data, EntitySchemaDependencyLink):
                    configuration_item = configuration_item_data.to_dict()

                elif isinstance(configuration_item_data, WorkflowTaskSchemaDependencyLink):
                    configuration_item = configuration_item_data.to_dict()

                elif isinstance(configuration_item_data, DropdownDependencyLink):
                    configuration_item = configuration_item_data.to_dict()

                elif isinstance(configuration_item_data, ResourceDependencyLink):
                    configuration_item = configuration_item_data.to_dict()

                elif isinstance(configuration_item_data, ScalarConfig):
                    configuration_item = configuration_item_data.to_dict()

                else:
                    configuration_item = configuration_item_data.to_dict()

                configuration.append(configuration_item)

        field_dict: Dict[str, Any] = {}
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if description is not UNSET:
            field_dict["description"] = description
        if name is not UNSET:
            field_dict["name"] = name
        if configuration is not UNSET:
            field_dict["configuration"] = configuration

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_description() -> Union[Unset, str]:
            description = d.pop("description")
            return description

        try:
            description = get_description()
        except KeyError:
            if strict:
                raise
            description = cast(Union[Unset, str], UNSET)

        def get_name() -> Union[Unset, str]:
            name = d.pop("name")
            return name

        try:
            name = get_name()
        except KeyError:
            if strict:
                raise
            name = cast(Union[Unset, str], UNSET)

        def get_configuration() -> Union[
            Unset,
            List[
                Union[
                    SchemaDependencyLink,
                    EntitySchemaDependencyLink,
                    WorkflowTaskSchemaDependencyLink,
                    DropdownDependencyLink,
                    ResourceDependencyLink,
                    ScalarConfig,
                    SecureTextConfig,
                    UnknownType,
                ]
            ],
        ]:
            configuration = []
            _configuration = d.pop("configuration")
            for configuration_item_data in _configuration or []:

                def _parse_configuration_item(
                    data: Union[Dict[str, Any]]
                ) -> Union[
                    SchemaDependencyLink,
                    EntitySchemaDependencyLink,
                    WorkflowTaskSchemaDependencyLink,
                    DropdownDependencyLink,
                    ResourceDependencyLink,
                    ScalarConfig,
                    SecureTextConfig,
                    UnknownType,
                ]:
                    configuration_item: Union[
                        SchemaDependencyLink,
                        EntitySchemaDependencyLink,
                        WorkflowTaskSchemaDependencyLink,
                        DropdownDependencyLink,
                        ResourceDependencyLink,
                        ScalarConfig,
                        SecureTextConfig,
                        UnknownType,
                    ]
                    discriminator_value: str = cast(str, data.get("type"))
                    if discriminator_value is not None:
                        if discriminator_value == "aa_sequence":
                            configuration_item = ResourceDependencyLink.from_dict(data, strict=False)

                            return configuration_item
                        if discriminator_value == "boolean":
                            configuration_item = ScalarConfig.from_dict(data, strict=False)

                            return configuration_item
                        if discriminator_value == "box":
                            configuration_item = ResourceDependencyLink.from_dict(data, strict=False)

                            return configuration_item
                        if discriminator_value == "box_schema":
                            configuration_item = SchemaDependencyLink.from_dict(data, strict=False)

                            return configuration_item
                        if discriminator_value == "container":
                            configuration_item = ResourceDependencyLink.from_dict(data, strict=False)

                            return configuration_item
                        if discriminator_value == "container_schema":
                            configuration_item = SchemaDependencyLink.from_dict(data, strict=False)

                            return configuration_item
                        if discriminator_value == "custom_entity":
                            configuration_item = ResourceDependencyLink.from_dict(data, strict=False)

                            return configuration_item
                        if discriminator_value == "date":
                            configuration_item = ScalarConfig.from_dict(data, strict=False)

                            return configuration_item
                        if discriminator_value == "datetime":
                            configuration_item = ScalarConfig.from_dict(data, strict=False)

                            return configuration_item
                        if discriminator_value == "dna_oligo":
                            configuration_item = ResourceDependencyLink.from_dict(data, strict=False)

                            return configuration_item
                        if discriminator_value == "dna_sequence":
                            configuration_item = ResourceDependencyLink.from_dict(data, strict=False)

                            return configuration_item
                        if discriminator_value == "dropdown":
                            configuration_item = DropdownDependencyLink.from_dict(data, strict=False)

                            return configuration_item
                        if discriminator_value == "entity_schema":
                            configuration_item = EntitySchemaDependencyLink.from_dict(data, strict=False)

                            return configuration_item
                        if discriminator_value == "entry":
                            configuration_item = ResourceDependencyLink.from_dict(data, strict=False)

                            return configuration_item
                        if discriminator_value == "entry_schema":
                            configuration_item = SchemaDependencyLink.from_dict(data, strict=False)

                            return configuration_item
                        if discriminator_value == "float":
                            configuration_item = ScalarConfig.from_dict(data, strict=False)

                            return configuration_item
                        if discriminator_value == "folder":
                            configuration_item = ResourceDependencyLink.from_dict(data, strict=False)

                            return configuration_item
                        if discriminator_value == "integer":
                            configuration_item = ScalarConfig.from_dict(data, strict=False)

                            return configuration_item
                        if discriminator_value == "json":
                            configuration_item = ScalarConfig.from_dict(data, strict=False)

                            return configuration_item
                        if discriminator_value == "location":
                            configuration_item = ResourceDependencyLink.from_dict(data, strict=False)

                            return configuration_item
                        if discriminator_value == "location_schema":
                            configuration_item = SchemaDependencyLink.from_dict(data, strict=False)

                            return configuration_item
                        if discriminator_value == "mixture":
                            configuration_item = ResourceDependencyLink.from_dict(data, strict=False)

                            return configuration_item
                        if discriminator_value == "molecule":
                            configuration_item = ResourceDependencyLink.from_dict(data, strict=False)

                            return configuration_item
                        if discriminator_value == "plate":
                            configuration_item = ResourceDependencyLink.from_dict(data, strict=False)

                            return configuration_item
                        if discriminator_value == "plate_schema":
                            configuration_item = SchemaDependencyLink.from_dict(data, strict=False)

                            return configuration_item
                        if discriminator_value == "project":
                            configuration_item = ResourceDependencyLink.from_dict(data, strict=False)

                            return configuration_item
                        if discriminator_value == "registry":
                            configuration_item = ResourceDependencyLink.from_dict(data, strict=False)

                            return configuration_item
                        if discriminator_value == "request_schema":
                            configuration_item = SchemaDependencyLink.from_dict(data, strict=False)

                            return configuration_item
                        if discriminator_value == "result_schema":
                            configuration_item = SchemaDependencyLink.from_dict(data, strict=False)

                            return configuration_item
                        if discriminator_value == "rna_oligo":
                            configuration_item = ResourceDependencyLink.from_dict(data, strict=False)

                            return configuration_item
                        if discriminator_value == "rna_sequence":
                            configuration_item = ResourceDependencyLink.from_dict(data, strict=False)

                            return configuration_item
                        if discriminator_value == "run_schema":
                            configuration_item = SchemaDependencyLink.from_dict(data, strict=False)

                            return configuration_item
                        if discriminator_value == "secure_text":
                            configuration_item = SecureTextConfig.from_dict(data, strict=False)

                            return configuration_item
                        if discriminator_value == "text":
                            configuration_item = ScalarConfig.from_dict(data, strict=False)

                            return configuration_item
                        if discriminator_value == "workflow_task_schema":
                            configuration_item = WorkflowTaskSchemaDependencyLink.from_dict(
                                data, strict=False
                            )

                            return configuration_item
                        if discriminator_value == "workflow_task_status":
                            configuration_item = ResourceDependencyLink.from_dict(data, strict=False)

                            return configuration_item
                        if discriminator_value == "worklist":
                            configuration_item = ResourceDependencyLink.from_dict(data, strict=False)

                            return configuration_item

                        return UnknownType(value=data)
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        configuration_item = SchemaDependencyLink.from_dict(data, strict=True)

                        return configuration_item
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        configuration_item = EntitySchemaDependencyLink.from_dict(data, strict=True)

                        return configuration_item
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        configuration_item = WorkflowTaskSchemaDependencyLink.from_dict(data, strict=True)

                        return configuration_item
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        configuration_item = DropdownDependencyLink.from_dict(data, strict=True)

                        return configuration_item
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        configuration_item = ResourceDependencyLink.from_dict(data, strict=True)

                        return configuration_item
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        configuration_item = ScalarConfig.from_dict(data, strict=True)

                        return configuration_item
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        configuration_item = SecureTextConfig.from_dict(data, strict=True)

                        return configuration_item
                    except:  # noqa: E722
                        pass
                    return UnknownType(data)

                configuration_item = _parse_configuration_item(configuration_item_data)

                configuration.append(configuration_item)

            return configuration

        try:
            configuration = get_configuration()
        except KeyError:
            if strict:
                raise
            configuration = cast(
                Union[
                    Unset,
                    List[
                        Union[
                            SchemaDependencyLink,
                            EntitySchemaDependencyLink,
                            WorkflowTaskSchemaDependencyLink,
                            DropdownDependencyLink,
                            ResourceDependencyLink,
                            ScalarConfig,
                            SecureTextConfig,
                            UnknownType,
                        ]
                    ],
                ],
                UNSET,
            )

        benchling_app_create = cls(
            description=description,
            name=name,
            configuration=configuration,
        )

        return benchling_app_create

    @property
    def description(self) -> str:
        if isinstance(self._description, Unset):
            raise NotPresentError(self, "description")
        return self._description

    @description.setter
    def description(self, value: str) -> None:
        self._description = value

    @description.deleter
    def description(self) -> None:
        self._description = UNSET

    @property
    def name(self) -> str:
        if isinstance(self._name, Unset):
            raise NotPresentError(self, "name")
        return self._name

    @name.setter
    def name(self, value: str) -> None:
        self._name = value

    @name.deleter
    def name(self) -> None:
        self._name = UNSET

    @property
    def configuration(
        self,
    ) -> List[
        Union[
            SchemaDependencyLink,
            EntitySchemaDependencyLink,
            WorkflowTaskSchemaDependencyLink,
            DropdownDependencyLink,
            ResourceDependencyLink,
            ScalarConfig,
            SecureTextConfig,
            UnknownType,
        ]
    ]:
        if isinstance(self._configuration, Unset):
            raise NotPresentError(self, "configuration")
        return self._configuration

    @configuration.setter
    def configuration(
        self,
        value: List[
            Union[
                SchemaDependencyLink,
                EntitySchemaDependencyLink,
                WorkflowTaskSchemaDependencyLink,
                DropdownDependencyLink,
                ResourceDependencyLink,
                ScalarConfig,
                SecureTextConfig,
                UnknownType,
            ]
        ],
    ) -> None:
        self._configuration = value

    @configuration.deleter
    def configuration(self) -> None:
        self._configuration = UNSET



from typing import Union

from ..extensions import UnknownType
from ..models.analysis_dataset_ids import AnalysisDatasetIds
from ..models.file_ids import FileIds

AnalysisStepDataUpdate = Union[FileIds, AnalysisDatasetIds, UnknownType]
